C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPGRPH                                                          *
C *                                                                  *
C *  Routine of BANDPLOT                                             *
C *                                                                  *
C *  Usage......: Modification of band structure plot.               *
C *                                                                  *
C *  Authors....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 12.09.94/mbs, 01.07.97/mbs                         *
C *  Remarks....: Operation system specific implementation: Look for *
C *               "Windows NT", "MS-Windows", "VMS", or "AIX".       *
C *               Changed for use with PGPLOT library.               *
C *                                                                  *
C *  Called from: BANDPL                                             *
C *                                                                  *
C *  Calls......: BPAXES,BPBRDR,BPENER,BPFRMI,BPSYML,BPTICA,BPTICS,  *
C *               BPTRAP                                             *
C *               PGPLOT routines PGQID,PGSLS,PGPAGE,PGSVP,PGSWIN,   *
C *               PGOPEN,PGSLCT,PGPAP,PGCURS                         *
C *                                                                  *
C ********************************************************************

      SUBROUTINE BPGRPH(VPXOFF,VPYOFF,VPXFUL,VPYFUL,VPXSCT,VPXAXW,
     .                  VPYAXH,EMIN,EMAX,NSYM,IFAIL)
C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)

      INCLUDE 'sizes'
 
      PARAMETER (ILEFT=0,ICENTR=1,IRIGHT=2)
 
      DIMENSION    VPXSCT(MAXSYM+1),ITICSD(MAXSYM+1),TEXTXY(MAXTEX,2)
      DIMENSION    TXTANG(MAXTEX),ITXPOS(MAXTEX)
      REAL         CURSX,CURSY
      CHARACTER    CANSWR
      CHARACTER*80 CLABEL(MAXSYM+1),CXAXLB(MAXSYM),CYAXLB,CTEXT(MAXTEX),
     .             STRING
      LOGICAL      FBORDR,FFLINE,FSELEC(MAXSEL)
C
C NAG/GKS version.
C
C      REAL         DEVXMN,DEVXMX,DEVYMN,DEVYMX
C      CHARACTER    CDMSTR
C
C PGPLOT version.
C
      INTEGER      PGOPEN,PGCURS
 
      DATA FSELEC/MAXSEL*.FALSE./
 
      FBORDR=(NSYM .GT. 1)
      MARGIN=0
      VPXEND=VPXOFF+VPXAXW
      VPYEND=VPYOFF+VPYAXH

C
C Define GKS specific constants.
C
C      IWSID=1
C      IWSTYP=0
C      IDEVIC=1
C      ITRNFR=1
C
C Inquire device dimensions.
C
C      DEVXMN=0.0
C      DEVYMN=0.0
C      CALL GQDSP(IWSTYP,IERR,IUNITS,DEVXMX,DEVYMX,IRASX,IRASY)
C
C Set current linestyle.
C
C NAG/GKS version.
C
C      CALL J06YRF(1)
C
C PGPLOT version.
C
      CALL PGSLS(1)
      CALL PGQID(IWSID)
 
   10 WRITE(6,20)
   20 FORMAT(/,' Graph modification menu:',//,
     .         ' ( 1) Change energy range',/,
     .         ' ( 2) Add border',/,
     .         ' ( 3) Add energy axis tic marks and tic labels',/,
     .         ' ( 4) Add symmetry axes labels',/,
     .         ' ( 5) Add symmetry line and energy axis titles',/,
     .         ' ( 6) Add optional text (eg. symmetry labels)',/,
     .         ' ( 7) Mark fermi energy'/,
     .         ' ( 8) Change symmetry line lengths',/,
     .         ' ( 9) Clear and redraw',/,
     .         ' (10) Generate output file',/,
     .         ' (11) Save graph layout',/,
     .         ' (12) Load graph layout',//,
     .         ' ( 0) Quit program')
   30 WRITE(6,'(/,'' Selection: '',$)')
      READ(5,'(I2)',ERR=30) ISEL
      IF ((ISEL .LT. 0) .OR. (ISEL .GT. MAXSEL)) GO TO 30
C
C ---------------------------------------------------------------------
C Change energy range
C
      IF (ISEL .EQ. 1) THEN
        DO I=1,MAXSEL
          FSELEC(I)=.FALSE.
        END DO
        CALL BPENER(EMIN,EMAX)
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL BPAXES(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,NSYM)
        CALL BPBAND(VPXOFF,VPYOFF,VPXSCT,VPYAXH,EMIN,EMAX,NSYM,IFAIL)
C
C ---------------------------------------------------------------------
C Add border.
C
      ELSE IF ((ISEL .EQ. 2) .AND. (NSYM .EQ. 1)) THEN
        FSELEC(ISEL)=.TRUE.
        FBORDR=.TRUE.
        CALL BPBRDR(VPXOFF,VPYOFF,VPXAXW,VPYAXH,VPXEND,VPYEND)
C
C ---------------------------------------------------------------------
C Add energy axis tic marks and tic labels.
C
      ELSE IF (ISEL .EQ. 3) THEN
        FSELEC(ISEL)=.TRUE.
        ERANGE=EMAX-EMIN
        WRITE(6,40) EMIN,EMAX,ERANGE
   40   FORMAT(' Energy axis tic marking and labeling.',//,
     .         ' Minimum energy: ',F10.5,' eV',/,
     .         ' Maximum energy: ',F10.5,' eV',/,
     .         ' Energy range:   ',F10.5,' eV',//,
     .         ' Tic range settings apply to all axes.',//)
   50   WRITE(6,'('' Tic start energy:          '',$)')
        READ(5,'(F10.5)',ERR=50) ETCMIN
        IF (ETCMIN .LT. EMIN) ETCMIN=EMIN
   60   WRITE(6,'('' Tic end energy:            '',$)')
        READ(5,'(F10.5)',ERR=60) ETCMAX
        IF (ETCMAX .GT. EMAX) ETCMAX=EMAX
        IF (ETCMAX .LT. ETCMIN) GOTO 50
   70   WRITE(6,'('' Distance between two tics: '',$)')
        READ(5,'(F10.5)',ERR=70) TICDIS
        IF (TICDIS .LE. 0.0) GOTO 70
 
        WRITE(6,'(/,'' Left axis:'')')
        CALL BPTICS(ITICSD(1))
   80   WRITE(6,'('' Tics per label (<1: no labels): '',$)')
        READ(5,'(I2)',ERR=80) ITCPLB
 
        TICDS2=0.0
        IF (ITCPLB .GT. 0) THEN
          IF (TICDIS .GT. ERANGE) THEN
            TICDIS=ERANGE
            ITCPLB=1
          ENDIF
C
C NAG/GKS version.
C
C          TICDS2=TICDIS*DBLE(ITCPLB)
C
C PGPLOT version.
C
          TICDS2=TICDIS*REAL(ITCPLB)
        ENDIF
C
C Annotate axis.
C
        INUM=1
        THRU=VPXOFF
        CALL BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,ETCMAX,
     .              THRU,TICDIS,TICDS2,ITICSD(1),INUM)
C
C Step through each symmetry line, ask for tic decoration.
C
        INUM=0
        IF (NSYM .GT. 1) THEN
          DO I=2,NSYM
            THRU=VPXOFF+VPXSCT(I)
            WRITE(6,'(/,'' Symmetry line '',I2,'':'')') I
            CALL BPTICS(ITICSD(I))
            CALL BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .                  ETCMAX,THRU,TICDIS,TICDS2,ITICSD(I),INUM)
          END DO
        ENDIF
C
C If there is a right border, ask for tic decoration.
C
        IF (FBORDR) THEN
          THRU=VPXEND
          WRITE(6,'(/,'' Right border:'')')
          CALL BPTICS(ITICSD(NSYM+1))
          CALL BPTICA(VPXOFF,VPYOFF,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .                ETCMAX,THRU,TICDIS,TICDS2,ITICSD(NSYM+1),INUM)
        ENDIF
C
C Use a cheap trick for output of whole polyline buffer.
C
C NAG/GKS version.
C
C        CALL J06YAF(1.0,EMIN)
C        CALL GUWK(1,1) 
C
C ---------------------------------------------------------------------
C Add symmetry axes labels.
C
      ELSE IF (ISEL .EQ. 4) THEN
        FSELEC(ISEL)=.TRUE.
C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(0.0,1.0,0.0,1.0,MARGIN)
C        CALL J06WCF(0.0,1.0,0.0,1.0)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,1.0,0.0,1.0)
        CALL PGSVP(0.0,1.0,0.0,1.0)
        DO I=1,NSYM
   90     WRITE(6,'('' Symmetry line '',I2,'', left label: '',$)') I
          READ(5,'(A20)',ERR=90) CLABEL(I)
          PENY=VPYOFF*0.75
          IF (I .EQ. 1) THEN
            PENX=VPXOFF
            STRING=CLABEL(1)
C
C NAG/GKS version.
C
C            CALL BPTRAN(STRING,PENX,PENY,ILEFT)
C
C PGPLOT version.
C
            CALL BPTRAP(STRING,PENX,PENY,0.0,ILEFT)
          ELSE
            PENX=VPXOFF+VPXSCT(I)
            STRING=CLABEL(I)
C
C NAG/GKS version.
C            
C            CALL BPTRAN(STRING,PENX,PENY,ICENTR)
C
C PGPLOT version.
C
            CALL BPTRAP(STRING,PENX,PENY,0.0,ICENTR)
          ENDIF
        END DO
  100   WRITE(6,'('' Right border, label:          '',$)')
        READ(5,'(A20)',ERR=100) CLABEL(NSYM+1)
        PENX=VPXEND
        PENY=VPYOFF*0.75
        STRING=CLABEL(NSYM+1)
C
C NAG/GKS version.
C
C        CALL BPTRAN(STRING,PENX,PENY,IRIGHT)
C
C PGPLOT version
C
        CALL BPTRAP(STRING,PENX,PENY,0.0,IRIGHT)
C
C ---------------------------------------------------------------------
C Add axes titles.
C
      ELSE IF (ISEL .EQ. 5) THEN
        FSELEC(ISEL)=.TRUE.
C
C Set datarange and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(0.0,VPXFUL,0.0,VPYFUL,MARGIN)
C        CALL J06WCF(0.0,VPXFUL,0.0,VPYFUL)
C
C PGPLOT version.
C
        CALL PGSWIN(0.0,VPXFUL,0.0,VPYFUL)
        CALL PGSVP(0.0,VPXFUL,0.0,VPYFUL)
        PENY=0.75*VPYOFF
        DO I=1,NSYM
  110     WRITE(6,'('' Label of symmetry axis '',I2,'': '',$)') I
          READ(5,'(A20)',ERR=110) CXAXLB(I)
          PENX=VPXOFF+(VPXSCT(I+1)+VPXSCT(I))/2.0
          STRING=CXAXLB(I)
C
C NAG/GKS version.
C
C          CALL BPTRAN(STRING,PENX,PENY,ICENTR)
C
C PGPLOT version.
C
          CALL BPTRAP(STRING,PENX,PENY,0.0,ICENTR)
        END DO
 
  120   WRITE(6,'('' Vertical axis label:   '',$)')
        READ(5,'(A20)',ERR=120) CYAXLB
        STRING=CYAXLB
C
C NAG/GKS version.
C
C        PENX=0.1*VPXOFF
C        PENY=VPYEND*1.01       
C        CALL BPTRAN(STRING,PENX,PENY,ILEFT)
C
C PGPLOT version.
C
        PENX=0.2*VPXOFF
        PENY=VPYOFF+0.5*VPYAXH
        CALL BPTRAP(STRING,PENX,PENY,90.0,ICENTR)
C
C----------------------------------------------------------------------
C Add optional text (eq. symmetry labels).
C
      ELSEIF (ISEL .EQ. 6) THEN
        FSELEC(ISEL)=.TRUE.
C
C Set datarange and viewport.
C
C NAG/GKS version./begin comment out
C 
C        CALL J06WBF(0.0,VPXFUL,0.0,VPYFUL,MARGIN)
C        CALL J06WCF(0.0,VPXFUL,0.0,VPYFUL)
C
C Change cursor.
C
C        IPET=2
C        IOPMOD=0
C        IECHO=1
C        CALL GSLCM(IWSID,IDEVIC,IOPMOD,IECHO)
C        CALL GINLC(IWSID,IDEVIC,ITRNFR,0.5,0.5,IPET,DEVXMN,DEVXMX,
C     .             DEVYMN,DEVYMX,0,CDMSTR)
C        WRITE(6,'('' After typing of text, position text with '',
C     .            ''mouse.'')')
C
        ITXT=0
  130   WRITE(6,'('' Text: '',$)')
        READ(5,'(A80)') CTEXT(ITXT+1)
        IF (CTEXT(ITXT+1) .GT. ' ') THEN
          ITXT=ITXT+1
C
C NAG/GKS version.
C
C          CALL GRQLC(IWSID,IDEVIC,ISTAT,ITRFRM,CURSX,CURSY)
C          TEXTXY(ITXT,1)=DBLE(CURSX)
C          TEXTXY(ITXT,2)=DBLE(CURSY)
C          CALL BPTRAN(CTEXT(ITXT),TEXTXY(ITXT,1),TEXTXY(ITXT,2),ILEFT)
C
C PGPLOT version.
C
  140     WRITE(6,'('' Position (left=0, center=1, right=2): '',$)')
          READ(5,'(I2)',ERR=140) IPOS
          IF ((IPOS .LT. 0) .OR. (IPOS .GT. 2)) GOTO 140
          ITXPOS(ITXT)=IPOS

  150     WRITE(6,'('' Text angle: '',$)')
          READ(5,'(F10.5)',ERR=150) TXTANG(ITXT)

          WRITE(6,'('' Position mouse pointer, then type any key.'')')

          CURSX=0.5
          CURSY=0.5
          IERR=PGCURS(CURSX,CURSY,CANSWR)
          TEXTXY(ITXT,1)=CURSX
          TEXTXY(ITXT,2)=CURSY

          CALL BPTRAP(CTEXT(ITXT),TEXTXY(ITXT,1),TEXTXY(ITXT,2),
     .                TXTANG(ITXT),ITXPOS(ITXT))
          GOTO 130
        ENDIF
C
C----------------------------------------------------------------------
C Mark fermi energy.
C
      ELSE IF (ISEL .EQ. 7) THEN
        FSELEC(ISEL)=.TRUE.
  380   WRITE(6,'(/,'' Fermi energy: '',$)')
        READ(5,'(F10.5)',ERR=380) EFERMI
        IF ((EFERMI .LT. EMIN) .OR. (EFERMI .GT. EMAX)) GOTO 380
  410   WRITE(6,'('' Mark with line? (y/n) '',$)')
        READ(5,'(A1)') CANSWR
        IF (INDEX('YyNn',CANSWR) .EQ. 0) GOTO 410
        FFLINE=(INDEX('Yy',CANSWR) .GT. 0)
        CALL BPFRMI(VPXOFF,VPYOFF,VPXFUL,VPYFUL,VPXAXW,VPYAXH,VPXEND,
     .              VPYEND,EMIN,EMAX,EFERMI,FFLINE)
C
C ---------------------------------------------------------------------
C Change symmetry line lengths.
C
      ELSE IF (ISEL .EQ. 8) THEN
        IF (NSYM .GT. 1) THEN
          CALL BPSYML(VPXSCT,VPXAXW,NSYM)
C
C NAG/GKS version.
C
C          CALL J06WDF
C
C PGPLOT version.
C
          CALL PGPAGE
          CALL BPRDRW(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,VPXEND,VPYEND,
     .                VPXFUL,VPYFUL,EMIN,EMAX,ERANGE,EFERMI,ETCMIN,
     .                ETCMAX,TICDIS,TICDS2,TEXTXY,TXTANG,
     .                NSYM,ITICSD,ITXT,ITXPOS,
     .                CLABEL,CXAXLB,CYAXLB,CTEXT,FBORDR,FFLINE,FSELEC)
        ENDIF
C
C ---------------------------------------------------------------------
C Clear and redraw.
C
      ELSE IF (ISEL .EQ. 9) THEN
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL BPRDRW(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,VPXEND,
     .              VPYEND,VPXFUL,VPYFUL,EMIN,EMAX,ERANGE,EFERMI,
     .              ETCMIN,ETCMAX,TICDIS,TICDS2,TEXTXY,TXTANG,
     .              NSYM,ITICSD,ITXT,ITXPOS,
     .              CLABEL,CXAXLB,CYAXLB,CTEXT,FBORDR,FFLINE,FSELEC)
C
C----------------------------------------------------------------------
C Generate output file (metafile, HPGL or postscript).
C
      ELSE IF (ISEL .EQ. 10) THEN
C
C NAG/GKS version.
C
C        WRITE(6,470)
C470     FORMAT(/,' Output file options:',/,
C     .           ' (1) Postscript',/, 
C     .           ' (2) Encapsulated Postscript',/, 
C     .           ' (3) Computer graphics metafile (CGM)', /)
C480     WRITE(6,'('' Selection: '',$)')
C        READ(5,'(I1)',ERR=480) IOUSEL
C        IF ((IOUSEL .LT. 1) .OR. (IOUSEL .GT. 3)) GOTO 480
C
C Deactivate screen output workstation.
C Open and activate output file workstation.
C
C        CALL GDAWK(IWSID)
C        IWSID=2
C        IF (IOUSEL .EQ. 1) THEN
C          IF (NSYM .LE. 3) THEN
C            CALL GOPWK(IWSID,IPSCON,IPSA4P)
C          ELSE
C            CALL GOPWK(IWSID,IPSCON,IPSA4L)
C          ENDIF
C        ELSE IF (IOUSEL .EQ. 2) THEN
C          IF (NSYM .LE. 3) THEN
C            CALL GOPWK(IWSID,IEPSCON,IEPSA4P)
C          ELSE
C            CALL GOPWK(IWSID,IEPSCON,IEPSA4L)
C          ENDIF
C        ELSE IF (IOUSEL .EQ. 3) THEN
C          CALL GOPWK(IWSID,ICGCON,ICGM)
C        ENDIF
C        CALL GACWK(IWSID)
C
C PGPLOT version.
C
        WRITE(6,'(/,'' Output file options:'',/,
     .              '' (1) Postscript/encapsulated postscript'',/,
     .              '' (2) HPGL'',/)')
  480   WRITE(6,'('' Selection: '',$)')
        READ(5,'(I1)',ERR=480) IOUSEL
        IF ((IOUSEL .LT. 1) .OR. (IOUSEL .GT. 2)) GOTO 480

        IORIEN=2
        IF (NSYM .GT. 3) IORIEN=1
        IDEVPR=PGOPEN(SDEVIC(IORIEN,IOUSEL))
        CALL PGSLCT(IDEVPR)
        CALL PGPAP(PAPERW,ASPECT)
        
        CALL BPRDRW(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,VPXEND,
     .              VPYEND,VPXFUL,VPYFUL,EMIN,EMAX,ERANGE,EFERMI,
     .              ETCMIN,ETCMAX,TICDIS,TICDS2,TEXTXY,TXTANG,
     .              NSYM,ITICSD,ITXT,ITXPOS,
     .              CLABEL,CXAXLB,CYAXLB,CTEXT,FBORDR,FFLINE,FSELEC)
C
C Deactivate and close output file workstation.
C Activate screen output workstation and redraw graph.
C
C NAG/GKS version.
C
C        CALL GDAWK(IWSID)
C        CALL GCLWK(IWSID)
C        IWSID=1
C        CALL GACWK(IWSID)
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGSLCT(IWSID)
        CALL PGPAGE
        CALL BPRDRW(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,VPXEND,
     .              VPYEND,VPXFUL,VPYFUL,EMIN,EMAX,ERANGE,EFERMI,
     .              ETCMIN,ETCMAX,TICDIS,TICDS2,TEXTXY,TXTANG,
     .              NSYM,ITICSD,ITXT,ITXPOS,
     .              CLABEL,CXAXLB,CYAXLB,CTEXT,FBORDR,FFLINE,FSELEC)
C
C ---------------------------------------------------------------------
C Save graph layout
C
      ELSE IF (ISEL .EQ. 11) THEN
C
C MS-Windows and Windows NT version: Add FILE='fort.15' to OPEN statement.
C VMS and AIX version: Remove FILE='fort.15' from OPEN statement.
C
        OPEN(15,FORM='formatted',STATUS='unknown',FILE='fort.15')
C        OPEN(15,FORM='formatted',STATUS='unknown')
        REWIND 15
        WRITE(15,510) NSYM
510     FORMAT(I2)
        WRITE(15,520) (FSELEC(I),I=1,MAXSEL)
520     FORMAT(20L1)
        WRITE(15,530) VPXOFF,VPYOFF,VPXAXW,VPYAXH,VPXFUL,VPYFUL
530     FORMAT(6F10.5)
        WRITE(15,540) (VPXSCT(I),I=1,NSYM+1)
540     FORMAT(F10.5)
        WRITE(15,550) EMIN,EMAX,EFERMI
550     FORMAT(3F10.5)
        WRITE(15,560) ETCMIN,ETCMAX,TICDIS,TICDS2
560     FORMAT(4F10.5)
        WRITE(15,570) (ITICSD(I),I=1,NSYM+1)
570     FORMAT(20I1)
        WRITE(15,580) (CLABEL(I),I=1,NSYM+1)
        WRITE(15,580) (CXAXLB(I),I=1,NSYM)
        WRITE(15,580) CYAXLB
580     FORMAT(A80)
        WRITE(15,590) ITXT
590     FORMAT(I4)
        WRITE(15,600) (TEXTXY(I,1),TEXTXY(I,2),TXTANG(I),ITXPOS(I),
     .                CTEXT(I),I=1,ITXT)
600     FORMAT(3F10.5,I2,A80)
        WRITE(15,610) FBORDR,FFLINE
610     FORMAT(2L1)
        CLOSE(15)
C
C ---------------------------------------------------------------------
C Load graph layout
C
      ELSE IF (ISEL .EQ. 12) THEN
C
C MS-Windows version: Add FILE='fort.15' to OPEN statement.
C VMS and AIX version: Remove FILE='fort.15' from OPEN statement.
C
        OPEN(15,FORM='formatted',STATUS='old',ERR=620,
     .       FILE='fort.15')
C        OPEN(15,FORM='formatted',STATUS='old',ERR=620)
        READ(15,510) NSYM
        READ(15,520) (FSELEC(I),I=1,MAXSEL)
        READ(15,530) VPXOFF,VPYOFF,VPXAXW,VPYAXH,VPXFUL,VPYFUL
        READ(15,540) (VPXSCT(I),I=1,NSYM+1)
        READ(15,550) EMIN,EMAX,EFERMI
        READ(15,560) ETCMIN,ETCMAX,TICDIS,TICDS2
        READ(15,570) (ITICSD(I),I=1,NSYM+1)
        READ(15,580) (CLABEL(I),I=1,NSYM+1)
        READ(15,580) (CXAXLB(I),I=1,NSYM)
        READ(15,580) CYAXLB
        READ(15,590) ITXT
        READ(15,600) (TEXTXY(I,1),TEXTXY(I,2),TXTANG(I),ITXPOS(I),
     .               CTEXT(I),I=1,ITXT)
        READ(15,610) FBORDR,FFLINE
        CLOSE(15)
        VPXEND=VPXOFF+VPXAXW
        VPYEND=VPYOFF+VPYAXH
        ERANGE=EMAX-EMIN
C
C NAG/GKS version.
C
C        CALL J06WDF
C
C PGPLOT version.
C
        CALL PGPAGE
        CALL BPRDRW(VPXOFF,VPYOFF,VPXSCT,VPXAXW,VPYAXH,VPXEND,
     .              VPYEND,VPXFUL,VPYFUL,EMIN,EMAX,ERANGE,EFERMI,
     .              ETCMIN,ETCMAX,TICDIS,TICDS2,TEXTXY,TXTANG,
     .              NSYM,ITICSD,ITXT,ITXPOS,
     .              CLABEL,CXAXLB,CYAXLB,CTEXT,FBORDR,FFLINE,FSELEC)
        GOTO 640
620     WRITE(6,'('' Layout file does not exist. '',
     .            ''First use option (11).'')')
640     CONTINUE
      ENDIF
      IF (ISEL .NE. 0) GOTO 10
 
      RETURN
      END