C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BPSELC                                                          *
C *                                                                  *
C *  Routine of BANDPLOT,                                            *
C *                                                                  *
C *  Usage......: Select bands for crossing.                         *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 19.08.92                                           *
C *  Revision...: 01.07.97/mbs                                       *
C *  Remarks....: Changed for use with PGPLOT graphics library.      *
C *                                                                  *
C *  Called from: BANDPL                                             *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSWIN,PGSVP,PGCURS                *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE BPSELC(X,EBUFF,EMIN,EMAX,XLEN,VPXOFF,VPYOFF,VPXAXW,
     .                  VPYAXH,NKLIN,NNOFIT,NBFIT,IBAND,KCHANG)
C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)

      INCLUDE 'sizes'
 
      DIMENSION X(MAXK),EBUFF(MAXBA,MAXK)
      REAL      PENX,PENY
C
C NAG/GKS version.
C
C      REAL      DEVXMN,DEVXMX,DEVYMN,DEVYMX
C
C PGPLOT version.
C
      INTEGER   PGCURS
      CHARACTER DUMMY
C
C The ACCURAcy parameter determines the maximum relative offset between
C user supplied and buffered curve value.
C
      PARAMETER (ACCURA=0.02)
 
      XLO=0.0
      XHI=XLEN
      XDIR=XLEN
 
      IBAND=0
      KCHANG=0
      VPXEND=VPXOFF+VPXAXW
      VPYEND=VPYOFF+VPYAXH
      ERANGE=EMAX-EMIN
C
C NAG/GKS version.
C Define GKS specific constants.
C
C      IWSID=1
C      IWSTYP=0
C      IDEVIC=1
C      ITRNFR=0
C
C Inquire device dimensions.
C
C      DEVXMN=0.0
C      DEVYMN=0.0
C      CALL GQDSP(IWSTYP,IERR,IUNITS,DEVXMX,DEVYMX,IRASX,IRASY)
C
C Set equal datarange and viewport.
C
C NAG/GKS version.
C
C      MARGIN=0
C      CALL J06WBF(VPXOFF,VPXEND,VPYOFF,VPYEND,MARGIN)
C      CALL J06WCF(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C PGPLOT version.
C
      CALL PGSWIN(VPXOFF,VPXEND,VPYOFF,VPYEND)
      CALL PGSVP(VPXOFF,VPXEND,VPYOFF,VPYEND) 
C
C NAG/GKS version.
C Change cursor.
C
C      IPET=2
C      IOPMOD=0
C      IECHO=1
C
C 
C      PENX=SNGL(VPXOFF)
C      PENY=SNGL(VPYOFF)
C      CALL GSLCM(IWSID,IDEVIC,IOPMOD,IECHO)
C      CALL GINLC(IWSID,IDEVIC,ITRNFR,PENX,PENY,IPET,DEVXMN,DEVXMX,
C     .           DEVYMN,DEVYMX,0,CDMSTR)
C
C Request locator coordinate.
C
C NAG/GKS version.
C
C10    CALL GRQLC(IWSID,IDEVIC,ISTAT,ITRNFR,PENX,PENY)
C      EFOUND=(DBLE(PENY)-VPYOFF)/VPYAXH*ERANGE+EMIN
C      XFOUND=XLO+(DBLE(PENX)-VPXOFF)/VPXAXW*XDIR
C
C PGPLOT version.
C
      WRITE(6,'(/,'' Put mouse pointer to chosen marker, then type '',
     .      ''a key.'',/)')

10    IRET=PGCURS(PENX,PENY,DUMMY)
      EFOUND=(PENY-VPYOFF)/VPYAXH*ERANGE+EMIN
      XFOUND=XLO+(PENX-VPXOFF)/VPXAXW*XDIR
      IF ((EFOUND .LT. EMIN) .OR. (EFOUND .GT. EMAX)) GOTO 10
      IF ((XFOUND .LT. 0.0) .OR. (XFOUND .GT. XLEN)) GOTO 10
C
C First determine the k-location where the user has clicked.
C
      XCMPLO=XFOUND-ACCURA*XLEN
      XCMPHI=XFOUND+ACCURA*XLEN
 
      DO K=1,NKLIN
        IF ((X(K) .GE. XCMPLO) .AND. (X(K) .LE. XCMPHI)) THEN
          KCHANG=K
          GOTO 20
        ENDIF
      END DO
      WRITE(6,'('' You didn''''t click near a marker.'')')
      GOTO 10
C
C k location found, step now through energies.
C
20    ECMPLO=EFOUND-ACCURA*ERANGE
      ECMPHI=EFOUND+ACCURA*ERANGE
 
      DO I=NNOFIT+1,NNOFIT+NBFIT
        ESEARC=EBUFF(I,KCHANG)
        IF ((ESEARC .LE. ECMPHI) .AND. (ESEARC .GE. ECMPLO)) THEN
          IBAND=I
          GOTO 30
        ENDIF
      END DO
      WRITE(6,'('' Band not found, try again.'')')
 
30    RETURN
      END
