C ********************************************************************
C * BANDPLOT - Plotting and interpolation of energy bands            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DOS                                                             *
C *                                                                  *
C *  Routine of BFIT (BANDPLOT)                                      *
C *                                                                  *
C *  Usage......: Does the actual interpolation of the bands.        *
C *                                                                  *
C *  Authors....: Sunil Wijeyesekera, Chong Zheng                    *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: -                                                  *
C *  Revision...: 17.02.92/mbs (Martin Braendle), 20.08.92/mbs,      *
C *               27.04.93/mbs, 01.07.97/mbs                         *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: BFIT                                               *
C *                                                                  *
C *  Calls......: INV,A,DA                                           *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE DOS(IND,INFD,R,Q,T,TI,LAM,WORK,G,C,AEND,EV,MV,KIN,
     .               EIN,AS,NBFIT,NK,N4,NSYMME,INF,NDIM,NDX,IUNIT)

C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)
      INCLUDE 'sizes'
 
      PARAMETER(SIG=1.2,MAXBA1=MAXBA+1)
 
      DIMENSION IND(2,NK),INFD(NK),R(MAXBA1),Q(MAXPTS,NBFIT),T(N4,N4),
     .          TI(N4,N4),WORK(N4),G(MAXBA),C(MAXBA),AEND(MAXSTA,NBFIT),
     .          EV(NBFIT),EIN(2*MAXK,MAXBA),AS(MAXSTA,N4),PMESH(MAXPTS)
C
C NAG/GKS version.
C 
C      DOUBLE PRECISION LAM(N4),MV(MAXSTA),KIN(MAXK),KI,KJ
C
C PGPLOT version.
C
      REAL LAM(N4),MV(MAXSTA),KIN(MAXK),KI,KJ
 
      TWOPI=3.14159265359*2.0
      NPMESH=MAXPTS
C
C Input data for interpolation
C
      DO K=1,NK
        IND(1,K)=0
      END DO
 
      IND(2,1)=INFD(1)
      DO K=2,NK
        IND(2,K)=IND(2,K-1)+INFD(K)
      END DO
 
      IF (IND(2,NK) .NE. INF) THEN
        WRITE(6,'(''  Input error: Wrong number of derivatives '',
     .            ''dropped.'')')
        RETURN
      ENDIF
 
      RNON=1.0
C
C NAG/GKS version.
C
C      IF (NSYMME .NE. 0) RNON=DBLE(NSYMME)
C
C PGPLOT version.
C
      IF (NSYMME .NE. 0) RNON=FLOAT(NSYMME)
      DO M=1,MAXSTA
C
C NAG/GKS version.
C
C        MV(M)=DBLE(M)*TWOPI/RNON
C        C(M)=DBLE(M)
C
C PGPLOT version.
C
        MV(M)=FLOAT(M)*TWOPI/RNON
        C(M)=FLOAT(M)
      END DO

      DO M=1,NPMESH
C
C NAG/GKS version.
C
C        PMESH(M)=DBLE(M-1)/100.0
C
C PGPLOT version.
C
        PMESH(M)=FLOAT(M-1)/100.0
      END DO
C
C Calculate basis functions
C
      DO 10 K=1,NK
        KI=KIN(K)
        DO 10 IX=1,NDX
          I1=IX-1
          IF ((I1 .GT. 0) .AND. (INFD(K) .EQ. 1)) GOTO 10
          J1=I1*NK+K-IND(IX,K)
          DO M=1,MAXSTA
            IF (IX .EQ. 1) AS(M,J1)=A(MV,KI,MAXSTA,M)
            IF (IX .NE. 1) AS(M,J1)=DA(MV,KI,MAXSTA,M)
          END DO
   10 CONTINUE
      DO M=1,MAXSTA
        Y=-SIG*C(M)
        IF (Y .LT. -13.0) Y=-13.0
        Y=EXP(Y)
        G(M)=Y
      END DO
      DO J1=1,N4
        DO J2=J1,N4
          T(J1,J2)=0.0
          DO M1=1,MAXSTA
            M=MAXSTA+1-M1
            T(J1,J2)=T(J1,J2)+G(M)*AS(M,J1)*AS(M,J2)
            T(J2,J1)=T(J1,J2)
          END DO
        END DO
      END DO
 
      CALL INV(T,TI,WORK,N4)
C
C The interpolation function is calculated for each band.
C
      DO JB=1,NBFIT
        D=0.0
        DO I=1,NK
          DO J=1,NK
            D=D+T(I,J)
          END DO
        END DO
        EV(JB)=0.0
        DO I=1,NK
          DO J=1,N4
            EV(JB)=EV(JB)+T(I,J)*EIN(J,JB)
          END DO
        END DO
        EV(JB)=EV(JB)/D
        DO I=1,N4
          LAM(I)=0.0
          DO J=1,N4
            X=0.0
            IF (J .LE. NK) X=EV(JB)
            LAM(I)=T(I,J)*(EIN(J,JB)-X)+LAM(I)
          END DO
        END DO
C        WRITE(6,125) (LAM(I),I=1,N4)
C  125   FORMAT(/,' Lambda',//,(6G19.6))
C        WRITE(6,127) SIG,EV(JB)
C  127   FORMAT(//,' Sigma for fourier damping is ',F15.5,///,
C     .            ' 1st fouriercomponent is ',F15.5,//,
C     .            ' the further fourier components are ')
        DO M=1,MAXSTA
          AEND(M,JB)=0.0
          DO J=1,N4
            AEND(M,JB)=AEND(M,JB)+LAM(J)*AS(M,J)
          END DO
          AEND(M,JB)=AEND(M,JB)*G(M)
        END DO
C        WRITE(6,145) (AEND(I,JB),I=1,MAXSTA)
C  145   FORMAT(6F19.6)
        Y=0.0
        DO I=1,NK
          DD=0.0
          KI=KIN(I)
          X=EV(JB)
          DO M1=1,MAXSTA
            M=MAXSTA+1-M1
            X=X+A(MV,KI,MAXSTA,M)*AEND(M,JB)
            IF (NDIM .NE. 0) THEN
              DD=DD+DA(MV,KI,MAXSTA,M)*AEND(M,JB)
            ENDIF
          END DO
C          WRITE(6,155) X,DD
C  155     FORMAT(/,' Interpolated values are ',2F15.5)
          X=X-EIN(I,JB)
          Y=Y+X*X
          IF (NDIM .NE. 0) THEN
            DO L=1,NDIM
              INN=IND(L+1,I)
              DD=DD-EIN(I+L*NK-INN,JB)
              Y=Y+DD**2
            END DO
          ENDIF
        END DO
C        WRITE(6,185) X,DD
C  185   FORMAT(/,' Deviations are',2F16.5,/)
        Y=SQRT(Y)
C        WRITE(6,185) Y
      END DO
C
C Interpolation to NPMESH k points for NBFIT bands
C
      DO L=1,NPMESH
        KJ=PMESH(L)
        DO IB=1,NBFIT
          R(IB)=EV(IB)
        END DO
        DO M1=1,MAXSTA
          M=MAXSTA+1-M1
          S=A(MV,KJ,MAXSTA,M)
          DO IB=1,NBFIT
            R(IB)=R(IB)+AEND(M,IB)*S
          END DO
        END DO
        DO IB=1,NBFIT
          Q(L,IB)=R(IB)
        END DO
      END DO
      IF (NSYMME .NE. 0) NPMESH=(MAXPTS+1)/2
      WRITE(IUNIT,'(2I3)') NPMESH,NBFIT
      WRITE(IUNIT,'(F10.5)') ((Q(L,IB),L=1,NPMESH),IB=1,NBFIT)
      NPMESH=MAXPTS
      RETURN
      END