C ********************************************************************
C * DOSPLOT -  Plots density of states (DOS)                         *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DPAXES                                                          *
C *                                                                  *
C *  Routine of DOSPLOT                                              *
C *                                                                  *
C *  Usage......: Annotates axes with tic labels and marks.          *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 12.02.93/mbs, 18.06.97/mbs                         *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: DOSPLT,DPGRPH                                      *
C *                                                                  *
C *  Calls......: DPTICA                                             *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE DPAXES(VPXOFF,VPXDOO,VPXDOW,VPXCOO,VPXCOW,VPYOFF,
     .                  VPXAXW,VPYAXH,
     .                  EMIN,EMAX,ETCMIN,ETCMAX,ETCDIS,ETCDS2,
     .                  XDOSMN,XDOSMX,DTCMIN,DTCMAX,DTCDIS,DTCDS2,
     .                  XCOPMX,CTCMIN,CTCMAX,CTCDIS,CTCDS2,IBUFDS,
     .                  IBUFCP,ITSDLF,ITSDMD,ITSDRT,ITSDUP,ITSDLO,
     .                  ITSCUP,ITSCLO,FARBTR)
C
C NAG/GKS version.
C 
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      LOGICAL FARBDS,FARBCP,FARBTR(2*MAXBUF)
 
      PARAMETER (IVERT=2,IHORIZ=1)
 
      VPXEND=VPXOFF+VPXAXW
      VPYEND=VPYOFF+VPYAXH
      ERANGE=EMAX-EMIN
      DRANGE=XDOSMX-XDOSMN
      CRANGE=2.0D0*XCOPMX
      XCOPMN=-XCOPMX
      FARBDS=.FALSE.
      FARBCP=.FALSE.
      DO I=1,MAXBUF
        IF (FARBTR(I)) FARBDS=.TRUE.
        IF (FARBTR(MAXBUF+I)) FARBCP=.TRUE.
      END DO
C
C Annotate left energy axis.
C
      IF (ITSDLF .GT. 0) THEN
        INUM=1
        THRU=VPXOFF
        CALL DPTICA(VPXOFF,VPYOFF,VPXAXW,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .              ETCMAX,THRU,ETCDIS,ETCDS2,ITSDLF,INUM,IVERT)
      ENDIF
C
C Annotate middle axis, if existent.
C
      IF (ITSDMD .GT. 0) THEN
        INUM=0
        IF (VPXDOO .EQ. VPXOFF) THEN
          THRU=VPXCOO
        ELSE
          THRU=VPXDOO
        ENDIF
        CALL DPTICA(VPXOFF,VPYOFF,VPXAXW,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .              ETCMAX,THRU,ETCDIS,ETCDS2,ITSDMD,INUM,IVERT)
      ENDIF
C
C Annotate right axis, if existent.
C
      IF (ITSDRT .GT. 0) THEN
        INUM=0
        THRU=VPXEND
        CALL DPTICA(VPXOFF,VPYOFF,VPXAXW,VPYAXH,EMIN,EMAX,ERANGE,ETCMIN,
     .              ETCMAX,THRU,ETCDIS,ETCDS2,ITSDRT,INUM,IVERT)
      ENDIF
C
C If only DOS plot exists, annotate it.
C
      IF ((IBUFDS .GT. 0) .AND. (IBUFCP .EQ. 0) .AND. .NOT. (FARBDS))
     . THEN
        IF (ITSDLO .GT. 0) THEN
          INUM=1
          THRU=VPYOFF
          CALL DPTICA(VPXDOO,VPYOFF,VPXDOW,VPYAXH,XDOSMN,XDOSMX,DRANGE,
     .                DTCMIN,DTCMAX,THRU,DTCDIS,DTCDS2,ITSDLO,INUM,
     .                IHORIZ)
        ENDIF
        IF (ITSDUP .GT. 0) THEN
          INUM=0
          THRU=VPYEND
          CALL DPTICA(VPXDOO,VPYOFF,VPXDOW,VPYAXH,XDOSMN,XDOSMX,DRANGE,
     .                DTCMIN,DTCMAX,THRU,DTCDIS,DTCDS2,ITSDUP,INUM,
     .                IHORIZ)
        ENDIF
C
C If only COOP plot exists, annotate it.
C
      ELSE IF ((IBUFDS .EQ. 0) .AND. (IBUFCP .GT. 0) .AND. .NOT.
     . (FARBCP)) THEN
        IF (ITSCLO .GT. 0) THEN
          INUM=1
          THRU=VPYOFF
          CALL DPTICA(VPXCOO,VPYOFF,VPXCOW,VPYAXH,XCOPMN,XCOPMX,CRANGE,
     .                CTCMIN,CTCMAX,THRU,CTCDIS,CTCDS2,ITSCLO,INUM,
     .                IHORIZ)
        ENDIF
        IF (ITSCUP .GT. 0) THEN
          INUM=0
          THRU=VPYEND
          CALL DPTICA(VPXCOO,VPYOFF,VPXCOW,VPYAXH,XCOPMN,XCOPMX,CRANGE,
     .                CTCMIN,CTCMAX,THRU,CTCDIS,CTCDS2,ITSCUP,INUM,
     .                IHORIZ)
        ENDIF
      ELSE
      ENDIF
C
C NAG/GKS version.
C
C      CALL GUWK(1,1)
      RETURN
      END