C ********************************************************************
C * DOSPLOT -  Plots density of states (DOS)                         *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  DPPLOT                                                          *
C *                                                                  *
C *  Routine of DOSPLOT                                              *
C *                                                                  *
C *  Usage......: Does plotting of DOS- or COOP curves.              *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 28.04.92                                           *
C *  Revision...: 02.09.94, 17.06.97/mbs                             *
C *  Remarks....: Major revision to PGPLOT graphics subroutine       *
C *               library.                                           *
C *                                                                  *
C *  Called from: DOSPLT,DPGRPH                                      *
C *                                                                  *
C *  Calls......: PGPLOT routines PGSLW,PGSWIN,PGSVP,PGSFS,PGPOLY,   *
C *                               PGSLS,PGLINE,PGMOVE,PGDRAW         *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE DPPLOT(VPXOFF,VPXWDT,VPYOFF,VPYAXH,EMIN,EMAX,ENERGY,
     .                  XMAX,XRANGE,DOSMN,DOSMX,COOPMX,DOFFST,
     .                  NMESH,IFINDX,IBUFDS,IBUFCP,IPLTTP,ILTYPE,ISHADE,
     .                  IBFNDX,STRING,BUFTIT,FARBTR,FCLOSE,FARBIT)
 
C
C NAG/GKS version.
C
C      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
C
C PGPLOT version.
C
      IMPLICIT REAL(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      PARAMETER (MARGIN=0)
 
      DIMENSION X(MXMESH),ENERGY(MXMESH),Y(MXMESH+2),DOFFST(MAXBUF)
      DIMENSION ILTYPE(2*MAXBUF),ISHADE(2*MAXBUF),IBFNDX(2*MAXBUF)
      CHARACTER*80 CDUMMY
      CHARACTER*72 BUFTIT(2*MAXBUF),STRING
      LOGICAL FARBIT,FREDRW,FARBTR(2*MAXBUF),FCLOSE(2*MAXBUF)
C
C NAG/GKS version.
C
C      REAL WLINE
C
      COMMON /BUF/BUFFER(2*MAXBUF*MXMESH),BUFMAX(2*MAXBUF),
     .            BUFMIN(2*MAXBUF)
 
      VPXEND=VPXOFF+VPXWDT
      VPYEND=VPYOFF+VPYAXH
C
C Set line width scale factor.
C NAG/GKS version (GKS function)
C
C      WLINE=1.0
C      CALL GSLWSC(WLINE)
C
C PGPLOT version.
C
      IWLINE=1
      CALL PGSLW(IWLINE)
     
C No new curve will be read, so all curves have to be redrawn.
C
      FREDRW=(IFINDX .EQ. 0)
 
      IF (IFINDX .GT. 0) THEN
        IF ((IPLTTP .EQ. 1) .AND. (IBUFDS .EQ. MAXBUF)) RETURN
        IF ((IPLTTP .EQ. 2) .AND. (IBUFCP .EQ. MAXBUF)) RETURN
 
        REWIND 1
        DO I=1,3*MXTITL*MXCARD+IFINDX*NMESH+4
          READ(1,'(A80)') CDUMMY
        END DO
 
        XMIN=1000.
C
C Read curve and inquire maximum and minimum of data range.
C
        DO I=1,NMESH
          READ(1,'(F12.5)') X(I)
          XMIN=MIN(X(I),XMIN)
          XMAX=MAX(X(I),XMAX)
        END DO
        IF (IPLTTP .EQ. 2) XMAX=MAX(ABS(XMIN),XMAX)
C
C Copy curve to buffer. Redraw of all curves must only be done if
C datarange has changed.
C
        IF (IPLTTP .EQ. 1) THEN
          IF ((XMAX .GT. DOSMX) .AND. (.NOT. FARBIT)) THEN
            DOSMX=XMAX
            FREDRW=.TRUE.
          ENDIF
          IF ((XMIN .LT. DOSMN) .AND. (.NOT. FARBIT)) THEN
            DOSMN=XMIN
            FREDRW=.TRUE.
          ENDIF
          IBUFDS=IBUFDS+1
          BUFMAX(IBUFDS)=XMAX
          BUFMIN(IBUFDS)=XMIN
          FARBTR(IBUFDS)=FARBIT
          XMAX=DOSMX
          XMIN=DOSMN
          DO I=1,NMESH
            BUFFER((IBUFDS-1)*NMESH+I)=X(I)
          END DO
          IBFNDX(IBUFDS)=IFINDX
          BUFTIT(IBUFDS)=STRING
          WRITE(6,'(/,1X,A72,/,'' assigned to DOS curve buffer #'',I2)') 
     .          STRING,IBUFDS
        ELSE
          IF ((XMAX .GT. COOPMX) .AND. (.NOT. FARBIT)) THEN
            COOPMX=XMAX
            FREDRW=.TRUE.
          ENDIF
          IBUFCP=IBUFCP+1
          BUFMAX(MAXBUF+IBUFCP)=XMAX
          BUFMIN(MAXBUF+IBUFCP)=-XMAX
          FARBTR(MAXBUF+IBUFCP)=FARBIT
          XMAX=COOPMX
          XMIN=-COOPMX
          DO I=1,NMESH
            BUFFER((MAXBUF+IBUFCP-1)*NMESH+I)=X(I)
          END DO
          IBFNDX(MAXBUF+IBUFCP)=IFINDX
          BUFTIT(MAXBUF+IBUFCP)=STRING
          WRITE(6,'(/,1X,A72,/,'' assigned to COOP curve buffer #'',
     .          I2)') STRING,IBUFCP+MAXBUF
        ENDIF
C
C Set data range and viewport.
C
C NAG/GKS version.
C
C        CALL J06WBF(XMIN*XRANGE,XMAX*XRANGE,EMIN,EMAX,MARGIN)
C
C PGPLOT version.
C
        CALL PGSWIN(XMIN*XRANGE,XMAX*XRANGE,EMIN,EMAX)
      ENDIF
C
C NAG/GKS version.
C
C      CALL J06WCF(VPXOFF,VPXEND,VPYOFF,VPYEND)
C
C PGPLOT version.
C
      CALL PGSVP(VPXOFF,VPXEND,VPYOFF,VPYEND)
 
      IF (FREDRW) THEN
        IF (IPLTTP .EQ. 1) THEN
C
C NAG/GKS version.
C
C          CALL J06WBF(DOSMN*XRANGE,DOSMX*XRANGE,EMIN,EMAX,MARGIN)
C
C PGPLOT version.
C
          CALL PGSWIN(DOSMN*XRANGE,DOSMX*XRANGE,EMIN,EMAX)
        ELSE
C
C NAG/GKS version.
C
C          CALL J06WBF(-COOPMX*XRANGE,COOPMX*XRANGE,EMIN,EMAX,MARGIN)
C
C PGPLOT version.
C
          CALL PGSWIN(-COOPMX*XRANGE,COOPMX*XRANGE,EMIN,EMAX)
        ENDIF
      ENDIF
C
C Draw buffered curve(s).
C
      ITYPE=1
      ISYM=1
 
      IF (IPLTTP .EQ. 1) THEN
        IHICNT=IBUFDS
        ILOCNT=1
      ELSE
        IHICNT=IBUFCP+MAXBUF
        ILOCNT=MAXBUF+1
      ENDIF
C 
C NAG/GKS version.
C
C      ESTEP=(ENERGY(NMESH)-ENERGY(1))/DBLE(NMESH-1)
C
C PGPLOT version.
C
      ESTEP=(ENERGY(NMESH)-ENERGY(1))/REAL(NMESH-1)
 
      DO I=IHICNT,ILOCNT,-1
C
C If closing of curve is requested, add two energy mesh points.
C
        JSTART=1
        JEND=NMESH
        NMESH2=NMESH
        IF (FCLOSE(I)) THEN
          JSTART=2
          JEND=NMESH+1
          NMESH2=NMESH+2
          X(1)=0.0
          X(NMESH2)=0.0
          Y(1)=ENERGY(1)
          Y(NMESH2)=ENERGY(NMESH)
        ENDIF
C
C Copy buffered curve to plot array.
C
        DO J=JSTART,JEND
          X(J)=BUFFER((I-1)*NMESH+J-JSTART+1)+DOFFST(I)
          Y(J)=ENERGY(J-JSTART+1)
        END DO
        IF (FARBTR(I)) THEN
C
C NAG/GKS version.
C
C          CALL J06WBF(BUFMIN(I)*XRANGE,BUFMAX(I)*XRANGE,EMIN,EMAX,
C     .                MARGIN)
C
C PGPLOT version.
C
           CALL PGSWIN(BUFMIN(I)*XRANGE,BUFMAX(I)*XRANGE,EMIN,EMAX)
        ELSE
          IF (FREDRW) THEN
            IF (IPLTTP .EQ. 1) THEN
C
C NAG/GKS version.
C
C              CALL J06WBF(DOSMN*XRANGE,DOSMX*XRANGE,EMIN,EMAX,MARGIN)
C
C PGPLOT version.
C
              CALL PGSWIN(DOSMN*XRANGE,DOSMX*XRANGE,EMIN,EMAX)
            ELSE
C
C NAG/GKS version.
C
C              CALL J06WBF(-COOPMX*XRANGE,COOPMX*XRANGE,EMIN,EMAX,MARGIN)
C
C PGPLOT version.
C
              CALL PGSWIN(-COOPMX*XRANGE,COOPMX*XRANGE,EMIN,EMAX)
            ENDIF
          ELSE
C
C NAG/GKS version.
C
C            CALL J06WBF(XMIN*XRANGE,XMAX*XRANGE,EMIN,EMAX,MARGIN)
C
C PGPLOT version.
C
            CALL PGSWIN(XMIN*XRANGE,XMAX*XRANGE,EMIN,EMAX)
          ENDIF
        ENDIF
C
C Shade area below curve if requested. Use NAG hatch fill brushes,
C earlier defined in routine DOSPLT.
C
        IF ((FCLOSE(I)) .AND. (ISHADE(I) .GT. 0)) THEN
C
C NAG/GKS version.
C
C          CALL J06YPF(ISHADE(I))
C          CALL J06YNF(X,Y,NMESH2)
C
C PGPLOT version.
C
          CALL PGSFS(1)
          CALL PGSCI(ISHADE(I))
          CALL PGPOLY(NMESH2,X,Y) 
        ENDIF
C
C Select pen respectively the line style.
C
C NAG/GKS version.
C
C        CALL J06YMF(ILTYPE(I))
C        CALL J06BAF(X,Y,NMESH2,ITYPE,ISYM,IFAIL)
C
C PGPLOT version.
C
        CALL PGSCI(1)
        CALL PGSLS(ILTYPE(I))
        CALL PGLINE(NMESH2,X,Y)
      END DO
C
C Change back to solid line type.
C Set line width scale factor.
C
C NAG/GKS version.
C
C      CALL J06YMF(1)
C      WLINE=3.0
C      CALL GSLWSC(WLINE)
C
C PGPLOT version.
C
      CALL PGSCI(1)
      IWLINE=3
      CALL PGSLW(IWLINE)
      CALL PGSLS(1)        
C
C Draw zero axis if COOP is to be plotted.
C
      IF (IPLTTP .EQ. 2) THEN
C
C NAG/GKS version.
C
C        CALL J06YAF(0.0,EMIN)
C        CALL J06YCF(0.0,EMAX)
C
C PGPLOT version.
C
        CALL PGMOVE(0.0,EMIN)
        CALL PGDRAW(0.0,EMAX)
      ENDIF
C
C Draw axes and output polyline buffer.
C
C NAG/GKS version.
C
C      CALL J06WBF(0.0,1.0,0.0,1.0,MARGIN)
C      CALL J06YAF(0.0,1.0)
C      CALL J06YCF(0.0,0.0)
C      CALL J06YCF(1.0,0.0)
C      CALL J06YAF(0.0,0.0)
C      CALL GUWK(1,1)
C
C PGPLOT version.
C
      CALL PGSWIN(0.0,1.0,0.0,1.0)
      CALL PGMOVE(0.0,1.0)
      CALL PGDRAW(0.0,0.0)
      CALL PGDRAW(1.0,0.0)
      
      RETURN
      END