C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  ABFNS                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates AB functions.                           *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987                                          *
C *  Remarks....: Only works for principal quantum # <= 7            *
C *                                                                  *
C *  Called from: MOV                                                *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE ABFNS(A,B)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      DIMENSION A(30),B(30)
 
      COMMON /LOCLAP/SK1,SK2,RR,L1,L2,M,N1,N2,MAXCAL
 
      J=MAXCAL+1
      RHO1=0.5*(SK1+SK2)*RR
      RHO2=0.5*(SK1-SK2)*RR
      IF (DABS(RHO1) .GT. 165.0D0) GO TO 210
      IF (DABS(RHO2) .GT. 165.0D0) GO TO 210
      C=DEXP(-RHO1)
      A(1)=C/RHO1
      DO 10 I=2,J
   10 A(I)=(DBLE(I-1)*A(I-1)+C)/RHO1
      IX=J
      IR=INT(DABS(2.0D0*RHO2))
      IS=MIN0(IR+1,29)
      IF (RHO2) 20,180,20
   20 D=DEXP(RHO2)
      H=1.0/D
C
C  if the value of rho2 is too small the sinh must be obtained
C  by summing the infinite series rather than by addition of
C  two exponentials.
C
      R=D-H
      IF (DABS(R)-0.1D0) 30,60,60
   30 RA=RHO2
      RHO22=RHO2*RHO2
      T=RHO2
      DO 40 I=2,50,2
        T=T*RHO22/DBLE(I*I+I)
        RA=RA+T
        IF (T .LT. 1.D-30) GOTO 50
   40 CONTINUE
   50 CONTINUE
      R=RA+RA
C
C  as many successive b-functions are generated from b-0 by the
C  recurrence formulae as accuracy will permit.
C
   60 B(1)=R/RHO2
      DO 170 I=2,IX,IS
        IF (IR .EQ. 0) GOTO 110
        IL=IS-1
        IF (1 .GT. IL) GOTO 100
        DO 90 J=1,IL
          K=I+J-1
          IF ((-1)**K) 70,70,80
   70     B(K)=(R+DBLE(K-1)*B(K-1))/RHO2
          GO TO 90
   80     B(K)=-(D+H-DBLE(K-1)*B(K-1))/RHO2
   90   CONTINUE
  100   CONTINUE
  110   IN=I+IS-1
C
C  after the recurrence formulae have been applied an appropriate
C  number of times the next b-function is obtained by summation
C  of the infinite series.
C
        IF (IN-IX) 120,120,200
  120   IF ((-1)**IN) 150,150,130
  130   TR=RHO2
        B(IN)=-2.0*TR/DBLE(IN+1)
        DO 140 J=1,500
          TR=TR*RHO2*RHO2/DBLE((2*J)*(2*J+1))
          IF (DABS(TR/B(IN))-1.0D-7) 170,170,140
  140   B(IN)=B(IN)-2.0*TR/DBLE(IN+1+2*J)
  150   TR=1.0D0
        B(IN)=2.0*TR/DBLE(IN)
C
        DO 160 J=1,500
          TR=TR*RHO2*RHO2/DBLE((2*J)*(2*J-1))
          IF (DABS(TR/B(IN))-1.0D-7) 170,170,160
  160   B(IN)=B(IN)+2.0*TR/DBLE(IN+2*J)
  170 CONTINUE
C
C  if the argument is zero a separate formula must be used.
C
      GOTO 200
  180 DO 190 I=1,IX,2
        B(I)=2.0/DBLE(I)
  190 B(I+1)=0.0
  200 RETURN
 
  210 DO 220 I=1,20
        A(I)=0.0
  220 B(I)=0.0
      GOTO 200
 
      END