C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  ARRAYD                                                          *
C *                                                                  *
C *  Routine of EHMACC,                                              *
C *  version for VMS, VAX FORTRAN                                    *
C *                                                                  *
C *  Usage......: Defines a single one-dimensional array a to be di- *
C *               vided into several two- and one-dimensional arrays *
C *               which are used later.                              *
C *               The size of the array A approximately determines   *
C *               the overall size of the program EHMACC.            *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 29.04.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Dimension of array A can be individually set by    *
C *               NDIMMX, the maximum number of atomic orbitals per  *
C *               unit cell. Because many of the atoms used are of   *
C *               s,p type, the relation NDIMMX=4*NATTMX (maximum    *
C *               number of atoms per unit cell) approximately       *
C *               holds. NDIMMX and NATMMX can be set in the inclu-  *
C *               defile EHMPAR.                                     *
C *                                                                  *
C *  Called from: PART2                                              *
C *                                                                  *
C *  Calls......: CRYST,HUCKEL                                       *
C *                                                                  *
C ********************************************************************

      SUBROUTINE ARRAYD
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'   
      PARAMETER (NA=5*NDIMMX*NDIMMX+3*NDIMMX)

      DIMENSION A(NA)

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /PASS/NATOM,NDIM

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      ND=NDIM
      NQ=ND*ND

      I1=1
      I2=I1+NQ
      I3=I2+ND
      I4=I3+NQ
      I5=I4+ND
      I6=I5+NQ
      I7=I1
      I8=I3
      I9=I6+ND
      I10=I9+NQ

      IF (INDEX(KEYWRD,'BAND') .EQ. 0) THEN
        CALL HUCKEL(A(I2),A(I3),A(I4),A(I5),A(I6),A(I7),A(I8),ND,NATOM)
      ELSE
        CALL CRYST(A(I1),A(I2),A(I3),A(I4),A(I5),A(I6),A(I7),A(I8),
     .             A(I9),A(I10),ND)
      ENDIF
      RETURN
      END
