C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  BHMAT                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates H(k).                                   *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 18.05.92/mbs, 18.02.93/mbs (Martin      *
C *               Braendle, c/o Calzaferri group, Inst. f. Inorg.    *
C *               and Phys. Chem., Freiestr. 3, CH-3012 Berne)       *
C *  Remarks....: Distance dependent weighted Wolfsberg-Helmholz-    * 
C *               formula included. Definition for EHMO calculations *
C *               s. G.Calzaferri et al, J.Phys.Chem. 93(1989), 5366 *
C *               Individual (kappa,delta). 18.02.93/mbs             * 
C *                                                                  *
C *  Called from: CRYST                                              *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE BHMAT(S,H,SEXP,ND,FDIST)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION S(ND,ND),H(ND,ND),SEXP(ND,ND)
      LOGICAL   FDIST

      COMMON /ATOMPT/IATMPT(NDIMMX)
      COMMON /DISTK/CKAPPA,CDELTA,AKAPPA(NATMMX,NATMMX),
     .              ADELTA(NATMMX,NATMMX)

      REWIND 22
      READ(22)  
      READ(22) H 
      REWIND 22

      CON=1.0D0+CKAPPA

      IF (FDIST) THEN
        DO 20 I=1,ND
          IATOM1=IATMPT(I)
          DO 10 J=1,I
            IATOM2=IATMPT(J)
            CON=1.0D0+AKAPPA(IATOM1,IATOM2)
            FAC2=1.0D0-1.0D0/CON
            H(I,J)=H(I,J)*(S(I,J)/CON+FAC2*SEXP(I,J))
            IF (I .NE. J) THEN
              CON=1.0D0+AKAPPA(IATOM2,IATOM1)
              FAC2=1.0D0-1.0D0/CON
              H(J,I)=H(J,I)*(S(J,I)/CON+FAC2*SEXP(J,I))
            ENDIF
   10     CONTINUE
   20   CONTINUE
      ELSE
        FAC=1.0D0/CON-1.0D0
        H(1,1)=H(1,1)*(FAC+S(1,1))
        DO 40 I=2,ND
          H(I,I)=H(I,I)*(FAC+S(I,I))
          IL=I-1
          DO 30 J=1,IL
            H(I,J)=H(I,J)*S(I,J)
            H(J,I)=H(J,I)*S(J,I)
   30     CONTINUE
   40   CONTINUE
      ENDIF

      RETURN
      END
