C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  CCHOL                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Computes choleski decomposition of given complex   *
C *               positive definitematrix a.                         *
C *                                                                  *
C *               input data                                         *
C *                n --- order of matrix                             *
C *                nd -- dimension of array a (it can be             *
C *                      greater than or equal to n)                 *
C *                a --- given matrix                                *
C *                      it is supposed to be stored in the          *
C *                      following way.  diagonal elements,          *
C *                      being real, are stored on the diagonal,     *
C *                      real parts of offdiagonal elements          *
C *                      are stored in the lower triangle of a,      *
C *                      imag parts of the lower triangle are        *
C *                      stored in the upper triangle of a.          *
C *                                                                  *
C *                   exit information                               *
C *                                                                  *
C *                a --- complex elements of matrix l, defined by    *
C *                   a=l*l(h)                                       *
C *                      are stored in the same way as original      *
C *                      elements of a, that is real parts of the    *
C *                      lower triangle of l in the lower triangle   *
C *                      of a and the corresponding imag parts in    *
C *                      the upper triangle of a.                    *
C *                ifail --- is set to zero if the decomposition was *
C *                         succesful and to nonzero if              *
C *                         the matrix was not positive definite.    *
C *                                                                  *
C *  Author.....: s. CBORIS                                          *
C *  Date.......: 20.06.1974                                         *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: CBORIS                                             *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE CCHOL (N,ND,A,IFAIL)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      DIMENSION A(ND,ND)
C
C     suppose decomposition will ifail
C
      IFAIL=1
C
      DO 40 I=1,N
C
C     test for positive definitness
C    
        IF (A(I,I) .LE. 0.0D0) RETURN 
C
C      compute column i
C
        A(I,I)=DSQRT(A(I,I))
        IF (I .EQ. N) GO TO 40
        IA=I+1
        DO 10 J=IA,N
          A(J,I)=A(J,I)/A(I,I)
   10   A(I,J)=A(I,J)/A(I,I)
C
C     reduce remaining columns
C
        DO 30 K=IA,N
          A(K,K)=A(K,K)-A(K,I)*A(K,I)-A(I,K)*A(I,K)
          IF (K .EQ. N) GO TO 30
          KA=K+1
          DO 20 J=KA,N
            A(J,K)=A(J,K)-A(J,I)*A(K,I)-A(I,J)*A(I,K)
   20     A(K,J)=A(K,J)-A(I,J)*A(K,I)+A(J,I)*A(I,K)
   30   CONTINUE
   40 CONTINUE
      IFAIL=0
      RETURN
      END
