C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  CRYST                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates the band orbital energies and other     *
C *               band orbital related quantities as a function of   *
C *               wave vector k                                      *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 02.05.92/mbs (Martin Braendle c/o       *
C *               Calzaferri group. Inst. f. Inorg. and Phys. Chem.  *
C *               Freiestr. 3, CH-3012 Berne), 19.05.92/mbs,         *
C *               11.06.92/mbs, 25.09.92/mbs, 24.08.94/mbs           *
C *  Remarks....: Slight modifications of some COMMON blocks.        *
C *               Distance dependent Hij included. 19.05.92/mbs      *
C *               Inclusion of summation over occupied atomic levels *
C *               and calculation of electrostatic two-body repul-   *
C *               sion energy. Unit 1 output formatted. 11.06.92/mbs *
C *               Output of H(k) and S(k) added. 25.09.92/mbs        *
C *               File access of different operating systems has to  *
C *               be adjusted. Look for OPEN statements.             *
C *               Automatic geometry variation and integration of    *
C *               total electronic energy EEHMO. 24.08.94/mbs        *
C *                                                                  *
C *  Called from: ARRAYD                                             *
C *                                                                  *
C *  Calls......: EHFMO,SREST,BSMAT,BHMAT,CBORIS,GRAD,OMATBO,OMATPB, *
C *               OMATT2,BFMO,BPOP,GETKP,REPUL                       *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE CRYST(D,DD,F,FF,V,VV,A,B,SEXP,DKSEXP,NDIM)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'  
 
      DIMENSION D(NDIM,NDIM),DD(NDIM),F(NDIM,NDIM),FF(NDIM),
     .          V(NDIM,NDIM),VV(NDIM),A(NDIM,NDIM),B(NDIM,NDIM),
     .          SEXP(NDIM,NDIM),DKSEXP(NDIM,NDIM)
      LOGICAL   FDIST,FVARY,EDITO
 
      COMMON /AOOUT1/INDTS(NDIMMX)
      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM, ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),
     .              EXPD(NSYMMX),EXPD2(NSYMMX),
     .              EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),
     .              CF1(NSYMMX),CF2(NSYMMX),COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),
     .              X(NATMMX),Y(NATMMX),Z(NATMMX)
      COMMON /DELET/IND(NDIMMX),INDIM,IVAL,NORBE(NATMMX),NDEL
      COMMON /FMOIO/IFMOIN(NDIMMX,NFMOMX),IFMOO(NDIMMX,NFMOMX),
     .              NDIMFR(NFMOMX),IFMOP(11,NFMOMX),NFMO
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /KPIN/TWEI,IKPOPT(MAXOPT+1),NKP
      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,IREST,IRESTV
      COMMON /OPTV2/IHKB,IHKE,ISKB,ISKE
      COMMON /SHOUTP/ISOUT(NDIMMX),IHOUT(NDIMMX),ISKOUT(NDIMMX),
     .               IHKOUT(NDIMMX)
      COMMON /TEMP1/NORBEF(NATMMX)
 
      COMMON /AELCON/AOOCC(NATMMX,4),USECON(NATMMX)
      LOGICAL USECON
 
      COMMON /AOOUT2/INDT(NDIMMX,2)
      CHARACTER INDT*10
 
      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2
 
      COMMON /INDX/NCRYST,NCEL1,NCEL2,NCEL3,XK(3),WEIGHT,TWOPI,ND1,
     .             NGRAD,NFULL,CEL1,CEL2
      LOGICAL CEL1,CEL2
 
      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      COMMON/SYMANA/TRANSV(MXSYME,3),SYMOP(MXSYME,3,3),
     .              IPERMU(MXSYME,NATMMX,2),ISYMRE(MXSYME,NDIMMX),
     .              NLINOP(3),IPOINT(3,MXSYME),ILINE(3),NSYMEL,
     .              CSYMEL(MXSYME),FSYMOP(MXSYME),FSYMAN(MXSYME),FSYM
      LOGICAL   FSYMOP,FSYM,FSYMAN
      CHARACTER CSYMEL*2
 
      COMMON /VARGEO/VARINC(NVARMX),IVRLIN(NVARMX),ISTPMX(NVARMX),
     .               IVARTP(NVARMX),ISTEPS(NVARMX),IVARY,NVARY,
     .               NVARLN,NNCPL,LCOUPL(NVARMX+1)
      LOGICAL LCOUPL
 
      COMMON /SORBIT/CSORB(NDIMMX)
      INTEGER CSORB

      DATA PI /3.1415926535898D0/

      ND1=NDIM-1
      TWOPI=2.0D0*PI
      CEL1=(NCEL1 .EQ. 1)
      CEL2=(NCEL2 .EQ. 1)
C
C check options
C
      IAOPOP=INDEX(KEYWRD,'AOPOP')
      FDIST=(INDEX(KEYWRD,'DISDEPWH') .GT. 0)
      IEOUT=INDEX(KEYWRD,'EOUT')
      IFERMI=INDEX(KEYWRD,'FERMI')
      IFPOP=INDEX(KEYWRD,'FMOPOP')
      IOPOP=INDEX(KEYWRD,'OVLPOP')
      IPOL=INDEX(KEYWRD,'POLAR')
      IWIND=INDEX(KEYWRD,'WINDOW')
      FVARY=(INDEX(KEYWRD,'VARIATION') .GT. 0)
      EDITO=(INDEX(KEYWRD,'EDITOUT') .GT. 0)
C
C increase the window to include the gaussian tails
C 0.5 eV   corresponds to  limit=1.0-5 and delta=0.15 (see ehprop)
C
      IF (IWIND .NE. 0) THEN
        WSTART=WSTART-0.5D0
        WEND=WEND+0.5D0
      ENDIF
C
C limitation of output (eigen values and eigen vectors)
C
      IF (LER .GT. NDIM) LER=NDIM
      IF (LBR .GT. LER) LBR=LER
      IF (LBR .EQ. 0) LBR=1
      IF (LER .EQ. 0) LER=NDIM
      IF (LE .GT. NDIM) LE=NDIM
      IF (LB .GT. LE) LB=LE
      IF (LB .EQ. 0) LB=1
      IF (LE .EQ. 0) LE=NDIM
      IF (IEE .GT. NDIM) IEE=NDIM
      IF (IEB .GT. IEE) IEB=IEE
      IF (IEB .EQ. 0) IEB=1
      IF (IEE .EQ. 0) IEE=NDIM
C
C other limitations
C
      NFU=NELEC/2
      IF (NELEC .NE. NFU*2) NFU=NFU+1
C
C check option GRAD
C
      NGRAD=INDEX(KEYWRD,'GRAD')
C
C calculate FMO for the unit cell
C
      IF (NFMO .GT. 0) THEN
        CALL EHFMO(DD,FF,V,VV,A,B,NDIM,INATOM)
        WRITE(IOUT,10)
   10   FORMAT(///)
      ENDIF
 
      WRITE(IOUT,20)
   20 FORMAT(/,1X,131('-'),/,5X,'Band calculation',/)
C
C Restart
C
      IF (INDEX(KEYWRD,'RESTART') .NE. 0) THEN
        IF (IVARY .EQ. 1) THEN
          OPEN(UNIT=1,FORM='formatted',STATUS='unknown')
          REWIND 1
          IF (IAOPOP .NE. 0) THEN
            OPEN(UNIT=2,FORM='formatted',STATUS='unknown')
            REWIND 2
          ENDIF
          IF (IOPOP .NE. 0) THEN
            OPEN(UNIT=3,FORM='formatted',STATUS='unknown')
            REWIND 3
          ENDIF
          IF (IFPOP .NE. 0) THEN
            OPEN(UNIT=4,FORM='formatted',STATUS='unknown')
            REWIND 4
          ENDIF
        ENDIF
        IF (INDEX(KEYWRD,'VARIATION') .GT. 0) THEN
          IF (IVARY .LT. IRESTV) THEN
            IREST=NKP+1
            CALL SREST(IREST,IAOPOP,IFPOP,IOPOP,IVARY,NKP,NDIM,INATOM,
     .                 NELEM,NDEL,NVARY)
          ELSE
            IREST=0
          ENDIF
        ELSE
          CALL SREST(IREST,IAOPOP,IFPOP,IOPOP,IVARY,NKP,NDIM,INATOM,
     .                 NELEM,NDEL,NVARY)
        ENDIF
      ENDIF
C
C prepare scratch file for energy integration
C
      IF (FVARY) REWIND 20
C
C loop over nkp
C
      DO 240 NCOUNT=1,NKP
        CLOSE(IOUT)
C
C Closing and opening done due to data security.
C VMS-version: add ACCESS='append' to OPEN statement.
C MS-DOS-version: add ACCESS='append' and FILE='fort.6' to OPEN
C                 statement.
C AIX-version: delete ACCESS='append' and FILE='fort.6' from OPEN
C              statement, compile with option -qposition=append
C
        OPEN(UNIT=IOUT,FORM='formatted',STATUS='old',ACCESS='append',
     .       FILE='fort.6')
        IBLAN=0
C
C get k point
C
        CALL GETKP(0)
        IF (FSYM) NGRAD=1
        IF (NCOUNT .LT. IREST) GOTO 240
        IF (WEIGHT .LT. 1.0E-10) WEIGHT=1.0
        WEIGHT=WEIGHT/TWEI
 
        WRITE(IOUT,30) NCOUNT,XK(1),XK(2),XK(3),WEIGHT
   30   FORMAT(1X,'kp #',I4,5X,3F17.13,10X,'normalized weight = ',G17.8)
C
C output for the terminal or the log file
C
        IF (NCOUNT .EQ. 1) WRITE(10,10)
        WRITE(10,40) NCOUNT,XK(1),XK(2),XK(3),WEIGHT
   40   FORMAT(' kp #',I3,4X,' k = ',3F10.6,5X,'weight : ',G15.5)
        IF (NCOUNT .EQ. NKP) WRITE(10,10)
 
        XK(1)=XK(1)*TWOPI
        XK(2)=XK(2)*TWOPI
        XK(3)=XK(3)*TWOPI
C
C calculate S(k) and store it in matrix F. If distance dependent
C Hij are to be calculated, each sum term of S(k) will be multiplied
C by exp(-CDELTA*(R-D0)), which will be stored in SEXP.
C 13.05.92/mbs
C
        CALL BSMAT(F,D,V,SEXP,DKSEXP,NDIM,FDIST)
        WRITE(23) F
        REWIND 23
C
C calculate H(k) and store it in matrix D
C
        CALL BHMAT(F,D,SEXP,NDIM,FDIST)
C
C output S(k)
C
        IF (IKPOPT(8) .NE. 0) THEN
          WRITE(IOUT,41)
   41     FORMAT(//,5X,'Overlap matrix S(K) of bloch orbitals',/,
     .           5X,'Lower triangle: real part, upper triangle: ',
     .           'imaginary part.',/)
          CALL OMATT2(F,NDIM,ISKB,ISKE,ISKB,ISKE,ISKOUT,ISKOUT,1,1)
        ENDIF
C
C output H(k)
C
        IF (IKPOPT(9) .NE. 0) THEN
          WRITE(IOUT,42)
   42     FORMAT(//,5X,'Hamiltonian matrix H(k) of bloch orbitals',/,
     .           5X,'Lower triangle: real part, upper triangle: ',
     .           'imaginary part.',/)
          CALL OMATT2(D,NDIM,IHKB,IHKE,IHKB,IHKE,IHKOUT,IHKOUT,1,1)
        ENDIF
C
C solve the eigenvalue problem
C
        CALL CBORIS(NDIM,NDIM,D,F,V,DD,FF,VV,IFAIL)

        IF (IFAIL .NE. 0) THEN
          SLINE='Orbitals too diffuse, more overlaps may be needed'
          CALL WRITEL(SLINE)
          RETURN
        ENDIF
C
C Analyze symmetry
C
        IF (FSYM .AND. (ILINE(1) .GT. 0)) CALL SYMMAN(D,V,NDIM)
C
C calculate occupation
C
        ILAST=0
        IF (IFERMI .EQ. 0) GOTO 70
        N3=NDIM
        N1=1
   50   N2=(N3+N1)/2
        IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 60
        IF (DD(N2) .GT. FERMI) THEN
          N3=N2
        ELSE
          N1=N2
        ENDIF
        GOTO 50
   60   ILAST=N2
        IF (DD(N2) .GT. FERMI) ILAST=ILAST-1
        IF (ILAST .LT. 1) ILAST=1
C
C determine the starting and end level of the window
C
   70   LEVS=0
        LEVE=0
        IF ((IOPOP .EQ. 0) .OR. (IWIND .EQ. 0)) GOTO 120
        N3=NDIM
        N1=1
   80   N2=(N3+N1)/2
        IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 90
        IF (DD(N2) .GT. WSTART) THEN
          N3=N2
        ELSE
          N1=N2
        ENDIF
        GOTO 80
   90   LEVS=N2
        IF (DD(N2) .GT. WSTART) LEVS=LEVS-1
        IF (LEVS .LT. 1) LEVS=1
 
        N3=NDIM
        N1=1
  100   N2=(N3+N1)/2
        IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 110
        IF (DD(N2) .GT. WEND) THEN
          N3=N2
        ELSE
          N1=N2
        ENDIF
        GOTO 100
  110   LEVE=N2
        IF (DD(N2) .LT. WEND) LEVE=LEVE+1
        IF (LEVE .GT. NDIM) LEVE=NDIM
 
  120   IF (NGRAD .NE. 0) GOTO 190
C
C print E between ieb and iee, if no gradients
C
        IF (IEOUT .EQ. 0) GOTO 180
        IBLAN=1
        WRITE(IOUT,130)
  130   FORMAT(//,5X,'Band orbital energies',/)
 
        DO 140 I=IEB,IEE
          WRITE(IOUT,150) I,DD(I)
  140   CONTINUE
  150   FORMAT(15X,I3,F12.6)
 
        IF (IFERMI .EQ. 0) GOTO 180
        TOTALE=0.0D0
        DO 160 I=1,ILAST
          TOTALE=TOTALE+DD(I)*2.0
  160   CONTINUE
        WRITE(IOUT,170) FERMI,ILAST,TOTALE
  170   FORMAT(/,5X,'For fermi level =',F12.6,',  ',I3,' levels are',
     .         ' doubly occupied',/,5X,'Total energy     = ',F12.4,
     .         ' eV',/)
  180   CONTINUE
C
C store E for plot or/and property
C
C
C VMS-version: delete FILE='fort.1' from OPEN statement.
C MS-DOS-version: add FILE='fort.1' to OPEN statement.
C AIX-version: delete FILE='fort.1' from OPEN statement.
C
  190   IF ((NCOUNT .EQ. 1) .AND. (IVARY .EQ. 1)) THEN
          OPEN (UNIT=1,FORM='formatted',STATUS='new',FILE='fort.1')
          WRITE(1,191) NKP,NDIM,INATOM,NELEM,NDEL,NVARY
  191     FORMAT(6I6)
          DO 193 I=1,NDIM
            WRITE(1,192) (INDT(I,J),J=1,2),IND(I)
  192       FORMAT(2A11,I6)
  193     CONTINUE
          DO 195 I=1,NELEM
            WRITE(1,194) SYMBOL(I),NS(I,1),NP(I),ND(I),NF(I)
  194       FORMAT(A3,4I2)
  195     CONTINUE
          DO 197 I=1,INATOM
            WRITE(1,196) AC(I),NORBEF(I),NORBE(I)
  196       FORMAT(A8,2I6)
  197     CONTINUE
          CLOSE(1)
        ENDIF
C
C Closing and opening done due to data security.
C VMS-version: add ACCESS='append', delete FILE='fort.1' from OPEN
C              statement.
C MS-DOS-version: add ACCESS='append' and FILE='fort.1' to OPEN
C                 statement.
C AIX-version: delete ACCESS='append' and FILE='fort.1' from OPEN
C              statement, compile with option -qposition=append
C
        OPEN(UNIT=1,FORM='formatted',STATUS='unknown',ACCESS='append',
     .       FILE='fort.1')
        WRITE(1,198) (XK(I),I=1,3),WEIGHT
  198   FORMAT(4F12.6)
        IF (FVARY) WRITE(20) WEIGHT
        DO 200 I=1,NDIM
          WRITE(1,199) DD(I)
  199     FORMAT(F12.6)
          IF (FVARY) WRITE(20) DD(I)
  200   CONTINUE
        CLOSE(1)



C
C store C after S(k) (d : real(C), v : imag(C))
C
        READ(23)
        WRITE(23) D
        WRITE(23) V
        REWIND 23
 
        IF (NGRAD .GT. 0) THEN
C
C Calculate de/dk and print de/dk along with E. Roughly outlined
C in M.Kertesz, T.Hughbanks, Smooth energy-band interpolation with
C gradient utilization, Phys. Rev. B 24(1981),6870. Original meaning of
C SEXP and DKSEXP is changed, because stored values are not needed any-
C more. SEXP serves as temporal reading matrix, DKSEXP will hold one
C part of the distance dependent dH(k)/dk values.
C 19.05.92/mbs
C
          REWIND 23
          CALL GRAD(D,DD,F,FF,V,VV,DKSEXP,SEXP,NDIM,ILAST,FDIST)
          IBLAN=1
        ENDIF

C
C print output for cEDiT
C
        IF ( EDITO ) THEN
          CALL OEDIT(NDIM,NCOUNT,XK,INATOM,IEB,IEE,DD,X,Y,Z,D,V,WEIGHT)
        ENDIF
 
        IF (IKPOPT(1) .EQ. 0) GOTO 230
        IOM=0
        DO 210 I=1,NDIM
          IOM=IOM+INDTS(I)
  210   CONTINUE
 
        IF (IOM .EQ. 0) GOTO 230
        IBLAN=1
C
C print orbital vectors
C
        WRITE(IOUT,220)
  220   FORMAT(//,5X,'Orbital vectors',/)
        IF (IPOL .EQ. 0) THEN
          CALL OMATBO(DD,D,V,NDIM,LBR,LER,LB,LE,INDTS,1)
        ELSE
          CALL OMATBP(DD,D,V,NDIM,LBR,LER,LB,LE,INDTS,1)
        ENDIF
C
C FMO population analysis
C
  230   IF ((IFPOP .NE. 0) .AND. (NFMO .GT. 0)) THEN
          CALL BFMO(D,F,FF,V,NDIM,NCOUNT,IBLAN,ILAST,LEVE)
        ENDIF
C
C population analysis
C
        IF ((IAOPOP .NE. 0) .OR. (IOPOP .NE. 0)) THEN
          CALL BPOP(D,DD,F,V,B,NDIM,NCOUNT,IBLAN,ILAST,LEVS,LEVE)
        ENDIF
C
C next k point
C
        IF (IBLAN .EQ. 1) WRITE(IOUT,10)
C
C end of loop over nkp
C
  240 CONTINUE
C
C Calculate electrostatic repulsion term and sum over Hii's over the
C occupied atomic levels.
C
      EAOORB=0.0D0
      EREP=0.0D0
      IF ((INDEX(KEYWRD,'EAOORB') .GT. 0) .OR.
     .    (INDEX(KEYWRD,'REPUL') .GT. 0)) THEN
        DO 250 I=1,INATOM
          KEYI=ISYM(I)
          EAOORB=EAOORB+AOOCC(I,1)*COULS(KEYI,1)+AOOCC(I,2)*COULP(KEYI)+
     .           AOOCC(I,3)*COULD(KEYI)+AOOCC(I,4)*COULF(KEYI)
  250   CONTINUE
        WRITE(IOUT,260) EAOORB
  260   FORMAT(/,2X,'Energies per unit cell [eV]:',//,
     .         2X,'Sum over occupied atomic energy levels   = ',F12.5)
      ENDIF
 
 
      IF (INDEX(KEYWRD,'REPUL') .NE. 0) THEN
        CALL REPUL(EREP)
        WRITE(IOUT,270) EREP
  270   FORMAT(2X,'Electrostatic core-core repulsion energy = ',F12.5)
      ENDIF
 
C
C Closing and opening done due to data security.
C VMS-version: add ACCESS='append' to, delete FILE='fort.1' from OPEN
C              statement.
C MS-DOS-version: add ACCESS='append' and FILE='fort.1' to OPEN
c                 statement.
C AIX-version: delete ACCESS='append' and FILE='fort.1' from OPEN
C              statement, compile with option -qposition=append .
C
      OPEN(UNIT=1,FORM='formatted',STATUS='unknown',ACCESS='append',
     .     FILE='fort.1')
 
      IF (FVARY) THEN
        CALL ETOT(EAOORB,EREP,NKP,NDIM)
      ELSE
        IORD=0
        WRITE(1,290) IORD
  290   FORMAT(I2)
      ENDIF
      CLOSE (1)
      RETURN
      END
