C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GETFMO                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Defines molecular fragments and obtain options for *
C *               FMO analysis.                                      *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 04.05.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: PART1                                              *
C *                                                                  *
C *  Calls......: ADDK,WRITEL,READA,SEARCH,READL                     *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE GETFMO
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      INCLUDE 'sizes'
 
      DIMENSION    ITEMP(NDIMMX)
      CHARACTER*10 TEXTF(20),TEXT(8)
      CHARACTER*7  ELEM(NSYMMX)
      CHARACTER    LINE*80,TEX1*11,TEX2*11,MOT*12,BL
 
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /DELET/IND(NDIMMX),INDIM,IVAL,NORBE(NATMMX),NDEL
      COMMON /TEMP1/NORBEF(NATMMX)
      COMMON /FMOIO/IFMOIN(NDIMMX,NFMOMX),IFMOO(NDIMMX,NFMOMX),
     .              NDIMFR(NFMOMX),IFMOP(11,NFMOMX),NFMO
 
      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80
 
      IF ((INDEX(SLINE,'FMO') .NE. 0) .AND.
     .    (INDEX(KEYWRD,'FMO') .EQ. 0)) THEN
        MOT='FMO'
        CALL ADDK(MOT)
      ENDIF
      IF ((INDEX(SLINE,'FMO') .EQ. 0) .AND.
     .    (INDEX(KEYWRD,'FMO') .EQ. 0)) GOTO 350
 
      BL=' '
      LDIM=INDIM-NDEL
      IP1=0
      IF (INDEX(KEYWRD,'BAND') .NE. 0) IP1=1
 
      NFMO=0
      IF (INDEX(SLINE,'FMO') .EQ. 0) THEN
        LINE='Cards FMO expected.'
        CALL WRITEL(LINE)
        RETURN
      ENDIF
      WRITE(IOUT,10)
   10 FORMAT(' ')
 
   20 IF (INDEX(SLINE,'FMO') .EQ. 0) GOTO 320
      WRITE(IOUT,10)
      NFMO=NFMO+1
      IF (NFMO .GT. NFMOMX) THEN
        LINE='Max number of fragments exceeded.'
        CALL WRITEL(LINE)
        CALL CLEXIT
      ENDIF
      IFMOP(1,NFMO)=-1
      IFMOP(2,NFMO)=-1
      IFMOP(3,NFMO)=-1
      IFMOP(4,NFMO)=-1
      DO 30 I=5,11
        IFMOP(I,NFMO)=0
   30 CONTINUE
      ISORT=0
      DO 40 I=1,LDIM
        IFMOIN(I,NFMO)=0
        IFMOO(I,NFMO)=0
   40 CONTINUE
   50 IF (INDEX(SLINE,'FMOIN') .EQ. 0) GOTO 110
      IF (INDEX(SLINE,'FMOIN') .EQ. INDEX(SLINE,'FMOINT')) GOTO 110
      NUMF=INT(READA(SLINE,6,80))
      IF (NUMF .NE. NFMO) GOTO 300
      DO 60 I=1,LDIM
        ITEMP(I)=0
   60 CONTINUE
      CALL SEARCH(TEXTF,NN,ELEM,KK,NORBEF,IND,NORBE,ITEMP)
      DO 80 I=1,LDIM
        IF (NFMO .NE. 1) THEN
          DO 70 J=1,NFMO-1
            IF (IFMOIN(I,J) .EQ. 1) GOTO 80
   70     CONTINUE
        ENDIF
        IF (IFMOIN(I,NFMO) .EQ. 0) IFMOIN(I,NFMO)=ITEMP(I)
   80 CONTINUE
      NDFR=0
      DO 90 J=1,LDIM
        IF (IFMOIN(J,NFMO) .EQ. 1) NDFR=NDFR+1
   90 CONTINUE
      NDIMFR(NFMO)=NDFR
      CALL READL
      TEXTF(1)=' FMOIN  #'
      TEX1=' Orbital : '
      TEX2=' Element : '
      WRITE(IOUT,100) TEXTF(1),NFMO,TEX2,(ELEM(L),BL,L=1,KK),TEX1,
     .                (TEXTF(L),L=2,NN)
  100 FORMAT(1X,A10,I2,1X,118A)
      GOTO 50
 
  110 IF (INDEX(SLINE,'FMOOPT') .EQ. 0) GOTO 240
      NUMF=INT(READA(SLINE,6,80))
      IF (NUMF .NE. NFMO) GOTO 300
      TEXT(1)=' '
      IP=INDEX(SLINE,'EOUT')
      IF (IP .NE. 0) THEN
        TEXT(1)='EOUT'
        IFMOP(1,NFMO)=0
        IFMOP(2,NFMO)=0
      ENDIF
      IP=INDEX(SLINE,'EOUT=')
      IF (IP .NE. 0) THEN
        IP=IP+5
        I1=INT(READA(SLINE,IP,80))
  120   IP=IP+1
        IF (SLINE(IP:IP+1) .EQ. 'TO') GOTO 130
        IF (IP .GT. 130) GOTO 130
        GOTO 120
  130   IP=IP+2
        I2=INT(READA(SLINE,IP,80))
        IF (I1 .GT. I2) I1=I2
        IF (I2 .GT. NDIMFR(NFMO)) I2=NDIMFR(NFMO)
        IFMOP(1,NFMO)=I1
        IFMOP(2,NFMO)=I2
      ENDIF
      TEXT(2)=' '
      IP=INDEX(SLINE,'FMOOUT')
      IF (IP .NE. 0) THEN
        TEXT(2)='FMOOUT'
        ISORT=1
        IFMOP(3,NFMO)=0
        IFMOP(4,NFMO)=0
      ENDIF
      IP=INDEX(SLINE,'FMOOUT=')
      IF (IP .NE. 0) THEN
        IP=IP+6
        I3=INT(READA(SLINE,IP,80))
  140   IP=IP+1
        IF (SLINE(IP:IP+1) .EQ. 'TO') GOTO 150
        IF (IP .GT. 130) GOTO 150
        GOTO 140
  150   IP=IP+2
        I4=INT(READA(SLINE,IP,80))
        IF (I3 .GT. I4) I3=I4
        IF (I4 .GT. NDIMFR(NFMO)) I4=NDIMFR(NFMO)
        IFMOP(3,NFMO)=I3
        IFMOP(4,NFMO)=I4
      ENDIF
      TEXT(3)=' '
      IP=INDEX(SLINE,'AOOUT')
      IF (IP .NE. 0) THEN
        TEXT(3)='AOOUT'
        IFMOP(5,NFMO)=0
        IFMOP(6,NFMO)=0
      ENDIF
      IP=INDEX(SLINE,'AOOUT=')
      IF (IP .NE. 0) THEN
        IP=IP+6
        I5=INT(READA(SLINE,IP,80))
  160   IP=IP+1
        IF (SLINE(IP:IP+1) .EQ. 'TO') GOTO 170
        IF (IP .GT. 130) GOTO 170
        GOTO 160
  170   IP=IP+2
        I6=INT(READA(SLINE,IP,80))
        IF (I5 .GT. I6) I5=I6
        IF (I6 .GT. NDIMFR(NFMO)) I6=NDIMFR(NFMO)
        IFMOP(5,NFMO)=I5
        IFMOP(6,NFMO)=I6
      ENDIF
      TEXT(4)=' '
      IF (INDEX(SLINE,'FMOOVLP') .NE. 0 .AND. IP1 .EQ. 0) THEN
        IFMOP(7,NFMO)=1
        TEXT(4)='FMOOVLP'
      ENDIF
      TEXT(5)=' '
      TEXT(6)=' '
      IF (INDEX(SLINE,'FMOINT') .NE. 0 .AND. IP1 .EQ. 0) THEN
        IFMOP(9,NFMO)=1
        TEXT(6)='FMOINT'
      ENDIF
      TEXT(7)=' '
      IF (INDEX(SLINE,'MOFMO') .NE. 0 .AND. IP1 .EQ. 0) THEN
        IFMOP(10,NFMO)=1
        TEXT(7)='MOFMO'
      ENDIF
      TEXT(8)=' '
      NBL=0
      DO 200 I=4,7
  180   IF (TEXT(I) .NE. ' ') GOTO 200
        DO 190 J=I,7-NBL
          TEXT(J)=TEXT(J+1)
  190   CONTINUE
        TEXT(8-NBL)=' '
        NBL=NBL+1
        NB=8-NBL
        IF (NB .LE. I) GOTO 210
        GOTO 180
  200 CONTINUE
  210 IF (TEXT(1) .NE. ' ') THEN
        WRITE(IOUT,220) NFMO,TEXT(1),(IFMOP(K,NFMO),K=1,2)
  220   FORMAT(2X,'FMOOPT # ',I2,2X,A10,' from ',I3,' to ',I3)
      ENDIF
      IF (TEXT(2) .NE. ' ') THEN
        WRITE(IOUT,220) NFMO,TEXT(2),(IFMOP(K,NFMO),K=3,4)
      ENDIF
      IF (TEXT(3) .NE. ' ') THEN
        WRITE(IOUT,220) NFMO,TEXT(3),(IFMOP(K,NFMO),K=5,6)
      ENDIF
      IF (TEXT(4) .NE. ' ') THEN
        WRITE(IOUT,230) NFMO,(TEXT(K),K=4,8)
  230 FORMAT(2X,'FMOOPT # ',I2,2X,5(A10,1X))
      ENDIF
      CALL READL
      GOTO 110
 
  240 IF (ISORT .EQ. 1) THEN
        DO 250 I=1,LDIM
          IFMOO(I,NFMO)=1
  250   CONTINUE
      ENDIF
      IF (INDEX(SLINE,'FMOOUT') .EQ. 0) GOTO 300
      DO 260 I=1,LDIM
        IFMOO(I,NFMO)=0
  260 CONTINUE
  270 IF (INDEX(SLINE,'FMOOUT') .EQ. 0) GOTO 300
      NUMF=INT(READA(SLINE,7,80))
      IF (NUMF .NE. NFMO) GOTO 300
      DO 280 I=1,LDIM
        ITEMP(I)=0
  280 CONTINUE
      CALL SEARCH(TEXTF,NN,ELEM,KK,NORBEF,IND,NORBE,ITEMP)
      DO 290 I=1,LDIM
        IF (IFMOIN(I,NFMO) .EQ. 0) GOTO 290
        IF (IFMOO(I,NFMO) .EQ. 0) IFMOO(I,NFMO)=ITEMP(I)
  290 CONTINUE
      CALL READL
      TEXTF(1)=' FMOOUT #'
      TEX1=' orbital : '
      TEX2=' element : '
      WRITE(IOUT,100) TEXTF(1),NFMO,TEX2,(ELEM(L),L=1,KK),TEX1,
     .                (TEXTF(L),L=2,NN)
      GOTO 270
 
  300 WRITE(IOUT,310) NFMO,NDIMFR(NFMO)
  310 FORMAT(3X,'Fragment # ',I3,' : ',I3,' orbitals')
      GOTO 20
  320 CONTINUE
      IF (IP1 .EQ. 0) THEN
        NUMB7=0
        NUMB9=0
        DO 330 I=1,NFMO
          NUMB7=NUMB7+IFMOP(7,I)
          NUMB9=NUMB9+IFMOP(9,I)
  330   CONTINUE
        IF (NUMB7 .NE. 1) GOTO 340
        LINE='Option FMOOVLP present only for one fragment.'
        CALL WRITEL(LINE)
  340   IF (NUMB9 .NE. 1) GOTO 300
        LINE='Option FMOINT present only for one fragment.'
        CALL WRITEL(LINE)
      ENDIF
      WRITE(IOUT,10)
 
  350 CONTINUE
 
      RETURN
      END
