C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GETSYM                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Gets definition of symmetry elements for wavefunc- *
C *               tion symmetry analysis.                            *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Institute for Inorganic and Phy- *
C *               sical Chemistry, University of Berne, Freiestr. 3, *
C *               CH-3012 Berne                                      *
C *  Date.......: 11.03.93                                           *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from:                                                    *
C *                                                                  *
C *  Calls......: READL,READA,WRITEL,GETPER                          *
C *                                                                  *
C ********************************************************************

      SUBROUTINE GETSYM

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      CHARACTER CATOM*7

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM, ISYM(NATMMX),NELEM

      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),
     .              EXPD(NSYMMX),EXPD2(NSYMMX),
     .              EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),
     .              CF1(NSYMMX),CF2(NSYMMX),COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),
     .              X(NATMMX),Y(NATMMX),Z(NATMMX)

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON/SYMANA/TRANSV(MXSYME,3),SYMOP(MXSYME,3,3),
     .              IPERMU(MXSYME,NATMMX,2),ISYMRE(MXSYME,NDIMMX),
     .              NLINOP(3),IPOINT(3,MXSYME),ILINE(3),NSYMEL,
     .              CSYMEL(MXSYME),FSYMOP(MXSYME),FSYMAN(MXSYME),FSYM
      LOGICAL   FSYMOP,FSYM,FSYMAN
      CHARACTER CSYMEL*2

      PARAMETER (TWOPI=6.283185307D0)

      NSYMEL=0
C
C Initialize permutation list.
C
      DO 30 I=1,MXSYME
        FSYMOP(I)=.FALSE.
        DO 20 J=1,NATMMX
          DO 10 K=1,2
            IPERMU(I,J,K)=0
   10     CONTINUE
   20   CONTINUE
   30 CONTINUE
C
C Beginning of read loop.
C    
   40 IF (INDEX(SLINE,'SYMMOP') .EQ. 0) GOTO 350
      ISYMEL=INT(READA(SLINE,1,80))
      IF (ISYMEL .GT. NSYMEL) THEN
        NSYMEL=ISYMEL
      ENDIF
      IF (NSYMEL .GT. MXSYME) THEN
        SLINE=' Max. numbers of symmetry operations exceeded. '
        CALL WRITEL(SLINE)  
      ENDIF

      IS=ISTART(2)
      CSYMEL(ISYMEL)=SLINE(IS:IS+1)

      IF (INDEX(CSYMEL(ISYMEL),'C') .EQ. 1) THEN
C
C Rotation axis, get rotation angle.
C
        N=INT(READA(CSYMEL(ISYMEL),1,2))
        PHI=TWOPI/DBLE(N)
        IF ((N .EQ. 1) .OR. (N .EQ. 5) .OR. (N .GT. 6)) THEN
          WRITE(IOUT,50) CSYMEL(ISYMEL)
   50     FORMAT(/,' Rotation axis ',A2,' not allowed in extended ',
     .           'structures!')
        ELSE
C
C Get position and direction of rotation axis.
C
          IS=ISTART(4)
          IF (INDEX(SLINE(IS:IS+3),'ATOM') .GT. 0) THEN
            CALL GETNAM(5,CATOM,IT)
            DO 60 I=1,INATOM
              IF (CATOM .EQ. AC(I)) GOTO 70
   60       CONTINUE
C
C Atom not found, write error message and exit program.
C
            SLINE=' Rotation axis: Wrong atom '//CATOM//'. '
            CALL WRITEL(SLINE)

   70       TRANSV(ISYMEL,1)=X(I)
            TRANSV(ISYMEL,2)=Y(I)
            TRANSV(ISYMEL,3)=Z(I)
            WRITE(IOUT,80) ISYMEL,CATOM,X(I),Y(I),Z(I)
   80       FORMAT(' Symmetry operation #',I2,': Rotation axis at ',
     .             'atom ',A7,', coordinates (',3F8.4,')') 
            IPOS=7       
          ELSE
            TRANSV(ISYMEL,1)=READA(SLINE,ISTART(4),80)
            TRANSV(ISYMEL,2)=READA(SLINE,ISTART(5),80)
            TRANSV(ISYMEL,3)=READA(SLINE,ISTART(6),80)
            WRITE(IOUT,90) ISYMEL,(TRANSV(ISYMEL,I),I=1,3)
   90       FORMAT(' Symmetry operation #',I2,': Rotation axis at (',
     .             3F8.4,')')
            IPOS=8
          ENDIF
          
          IS=ISTART(IPOS)
          IF (INDEX(SLINE(IS:IS+3),'ATOM') .GT. 0) THEN
            IPOS2=IPOS+1
            CALL GETNAM(IPOS2,CATOM,IT)
            DO 100 I=1,INATOM
              IF (CATOM .EQ. AC(I)) GOTO 110
  100       CONTINUE
C
C Atom not found, write error message and exit program.
C
            SLINE=' Rotation axis: Wrong atom '//CATOM//'. '
            CALL WRITEL(SLINE)

  110       XDIR=X(I)-TRANSV(ISYMEL,1)
            YDIR=Y(I)-TRANSV(ISYMEL,2)
            ZDIR=Z(I)-TRANSV(ISYMEL,3)
          ELSE
            XDIR=READA(SLINE,ISTART(IPOS),80)-TRANSV(ISYMEL,1)
            YDIR=READA(SLINE,ISTART(IPOS+1),80)-TRANSV(ISYMEL,2)
            ZDIR=READA(SLINE,ISTART(IPOS+2),80)-TRANSV(ISYMEL,3)
          ENDIF

          R=SQRT(XDIR*XDIR+YDIR*YDIR+ZDIR*ZDIR)
          XDIR=XDIR/R
          YDIR=YDIR/R
          ZDIR=ZDIR/R
          WRITE(IOUT,120) XDIR,YDIR,ZDIR
  120     FORMAT('                         Direction (',3F8.4,')')
C
C Set up transposed rotation matrix (Glaeske, p. 90)
C 
          SYMOP(ISYMEL,1,1)=XDIR*XDIR+(YDIR*YDIR+ZDIR*ZDIR)*COS(PHI)
          SYMOP(ISYMEL,2,1)=XDIR*YDIR*(1.0D0-COS(PHI))-ZDIR*SIN(PHI)
          SYMOP(ISYMEL,3,1)=XDIR*ZDIR*(1.0D0-COS(PHI))+YDIR*SIN(PHI)
          SYMOP(ISYMEL,1,2)=XDIR*YDIR*(1.0D0-COS(PHI))+ZDIR*SIN(PHI)
          SYMOP(ISYMEL,2,2)=YDIR*YDIR+(XDIR*XDIR+ZDIR*ZDIR)*COS(PHI)
          SYMOP(ISYMEL,3,2)=YDIR*ZDIR*(1.0D0-COS(PHI))-XDIR*SIN(PHI)
          SYMOP(ISYMEL,1,3)=XDIR*ZDIR*(1.0D0-COS(PHI))-YDIR*SIN(PHI)
          SYMOP(ISYMEL,2,3)=YDIR*ZDIR*(1.0D0-COS(PHI))+XDIR*SIN(PHI)
          SYMOP(ISYMEL,3,3)=ZDIR*ZDIR+(XDIR*XDIR+YDIR*YDIR)*COS(PHI)

          FSYMOP(ISYMEL)=.TRUE.
          FSYM=.TRUE.
        ENDIF

      ELSE IF (INDEX(CSYMEL(ISYMEL),'M') .EQ. 1) THEN
C
C Definition by 1 coordinate and 1 normal vector.
C
        IPOS=4
        IS=ISTART(IPOS)
        IF (INDEX(SLINE(IS:IS+3),'ATOM') .GT. 0) THEN
          IPOS2=IPOS+1
          CALL GETNAM(IPOS2,CATOM,IT)
          DO 150 I=1,INATOM
            IF (CATOM .EQ. AC(I)) GOTO 160
  150     CONTINUE
C
C Atom not found, write error message and exit program.
C
          SLINE=' Mirror plane: Wrong atom '//CATOM//'. '
          CALL WRITEL(SLINE)

  160     TRANSV(ISYMEL,1)=X(I)
          TRANSV(ISYMEL,2)=Y(I)
          TRANSV(ISYMEL,3)=Z(I)
          IPOS=6
          WRITE(IOUT,170) ISYMEL,CATOM,X(I),Y(I),Z(I)
  170     FORMAT(' Symmetry operation #',I2,': Mirror plane at atom ',
     .           A7,', coordinates (',3F8.4,')')
        ELSE
          TRANSV(ISYMEL,1)=READA(SLINE,ISTART(4),80)
          TRANSV(ISYMEL,2)=READA(SLINE,ISTART(5),80)
          TRANSV(ISYMEL,3)=READA(SLINE,ISTART(6),80)
          IPOS=7
          WRITE(IOUT,180) ISYMEL,(TRANSV(ISYMEL,I),I=1,3)
  180     FORMAT(' Symmetry operation #',I2,': Mirror plane at ',
     .           'coordinates (',3F8.4,')')
        ENDIF

        IS=ISTART(IPOS)
        IF (INDEX(SLINE(IS:IS+5),'NORMAL') .GT. 0) THEN
          IPOS=IPOS+1
          IS=ISTART(IPOS)
          IF (INDEX(SLINE(IS:IS+3),'ATOM') .GT. 0) THEN
            IPOS2=IPOS+1
            CALL GETNAM(IPOS2,CATOM,IT)
            DO 190 I=1,INATOM
              IF (CATOM .EQ. AC(I)) GOTO 200
  190       CONTINUE
C
C Atom not found, write error message and exit program.
C
            SLINE=' Mirror plane: Wrong atom '//CATOM//'. '
            CALL WRITEL(SLINE)

  200       VNORMX=X(I)-TRANSV(ISYMEL,1) 
            VNORMY=Y(I)-TRANSV(ISYMEL,2) 
            VNORMZ=Z(I)-TRANSV(ISYMEL,3)
          ELSE 
            VNORMX=READA(SLINE,ISTART(IPOS),80)-TRANSV(ISYMEL,1) 
            VNORMY=READA(SLINE,ISTART(IPOS+1),80)-TRANSV(ISYMEL,2)
            VNORMZ=READA(SLINE,ISTART(IPOS+2),80)-TRANSV(ISYMEL,3)
          ENDIF
        ELSE
C
C Definition by 3 coordinates.
C
   
          IF (INDEX(SLINE(IS:IS+3),'ATOM') .GT. 0) THEN
            IPOS2=IPOS+1
            CALL GETNAM(IPOS2,CATOM,IT)
            DO 210 I=1,INATOM
              IF (CATOM .EQ. AC(I)) GOTO 220
  210       CONTINUE
C
C Atom not found, write error message and exit program.
C
            SLINE=' Mirror plane: Wrong atom '//CATOM//'. '
            CALL WRITEL(SLINE)

  220       DIR1X=X(I)-TRANSV(ISYMEL,1) 
            DIR1Y=Y(I)-TRANSV(ISYMEL,2) 
            DIR1Z=Z(I)-TRANSV(ISYMEL,3)
            IPOS=IPOS+2
          ELSE 
            DIR1X=READA(SLINE,ISTART(IPOS),80)-TRANSV(ISYMEL,1)
            DIR1Y=READA(SLINE,ISTART(IPOS+1),80)-TRANSV(ISYMEL,2) 
            DIR1Z=READA(SLINE,ISTART(IPOS+2),80)-TRANSV(ISYMEL,3)
            IPOS=IPOS+3
          ENDIF
          IS=ISTART(IPOS)
          IF (INDEX(SLINE(IS:IS+3),'ATOM') .GT. 0) THEN
            IPOS2=IPOS+1
            CALL GETNAM(IPOS2,CATOM,IT)
            DO 230 I=1,INATOM
              IF (CATOM .EQ. AC(I)) GOTO 240
  230       CONTINUE
C
C Atom not found, write error message and exit program.
C
            SLINE=' Mirror plane: Wrong atom '//CATOM//'. '
            CALL WRITEL(SLINE)

  240       DIR2X=X(I)-TRANSV(ISYMEL,1) 
            DIR2Y=Y(I)-TRANSV(ISYMEL,2) 
            DIR2Z=Z(I)-TRANSV(ISYMEL,3)
          ELSE 
            DIR2X=READA(SLINE,ISTART(IPOS),80)-TRANSV(ISYMEL,1)
            DIR2Y=READA(SLINE,ISTART(IPOS+1),80)-TRANSV(ISYMEL,2)
            DIR2Z=READA(SLINE,ISTART(IPOS+2),80)-TRANSV(ISYMEL,3)
          ENDIF
C
C Calculate normal vector.
C
          VNORMX=DIR1Y*DIR2Z-DIR1Z*DIR2Y
          VNORMY=DIR1Z*DIR2X-DIR1X*DIR2Z
          VNORMZ=DIR1X*DIR2Y-DIR1Y*DIR2X
        ENDIF
        R=SQRT(VNORMX*VNORMX+VNORMY*VNORMY+VNORMZ*VNORMZ)
        VNORMX=VNORMX/R
        VNORMY=VNORMY/R
        VNORMZ=VNORMZ/R
        WRITE(IOUT,250) VNORMX,VNORMY,VNORMZ
  250   FORMAT('                         Normal vector (',3F8.4,')') 
C
C Set up transposed mirror transformation matrix.
C
        SYMOP(ISYMEL,1,1)=1.0D0-2.0D0*VNORMX*VNORMX
        SYMOP(ISYMEL,2,1)=-2.0D0*VNORMX*VNORMY
        SYMOP(ISYMEL,3,1)=-2.0D0*VNORMX*VNORMZ 
        SYMOP(ISYMEL,1,2)=-2.D0*VNORMX*VNORMY
        SYMOP(ISYMEL,2,2)=1.0D0-2.D0*VNORMY*VNORMY
        SYMOP(ISYMEL,3,2)=-2.0D0*VNORMY*VNORMZ
        SYMOP(ISYMEL,1,3)=-2.0D0*VNORMX*VNORMZ 
        SYMOP(ISYMEL,2,3)=-2.0D0*VNORMY*VNORMZ
        SYMOP(ISYMEL,3,3)=1.0D0-2.D0*VNORMZ*VNORMZ

        FSYMOP(ISYMEL)=.TRUE.
        FSYM=.TRUE.

      ELSE IF ((INDEX(CSYMEL(ISYMEL),'1') .EQ. 1) .OR.
     .         (INDEX(CSYMEL(ISYMEL),'E') .EQ. 1))  THEN
C
C Identity operation.
C
        WRITE(IOUT,310) ISYMEL
  310   FORMAT(' Symmetry operation #',I2,': Identity ')
C
C Set up symmetry operation.
C         
        DO 330 I=1,3
          DO 320 J=1,3
            IF (I .EQ. J) THEN
              SYMOP(ISYMEL,I,J)=1.0D0
            ELSE
              SYMOP(ISYMEL,I,J)=0.0D0
            ENDIF
  320     CONTINUE
  330   CONTINUE
        FSYMOP(ISYMEL)=.TRUE.
        FSYM=.TRUE.
C
C Wrong symmetry operation.
C
      ELSE
        WRITE(IOUT,340) CSYMEL(ISYMEL)
  340   FORMAT(/,' Symmetry operation ',A2,' does not exist!',/)
      ENDIF
C
C Get permutation list of atoms.
C
      CALL GETPER(ISYMEL)
      CALL READL
      GOTO 40
C
C End of read loop.
C
  350 IF (.NOT. FSYM) THEN
        WRITE(IOUT,360)
  360   FORMAT(' *********************',//,
     .         ' Card SYMMOP expected.',//,
     .         ' *********************',/)
      ENDIF

      RETURN
      END
