C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  GRAD                                                            *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Computes the gradients for a given k of each band. *
C *               The gradients are exactly calculated for given     *
C *               eigenvalues and eigenvectors. The method for cal-  *
C *               culating gradients used here is limited to nonde-  *
C *               generate bands(except when k=0). It is roughly de- *
C *               scribed in M.Kertesz, T.Hughbanks, Phys. Rev. B 24 *
C *               (1981), 6870.                                      *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 21.08.91/mbs, 20.05.92/mbs              *
C *               (Martin Braendle c/o                               *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestrasse 3, CH-3012 Berne)                     *
C *  Remarks....: Linear combinations of b(r) and b(-r) have been    *
C *               taken at the end of movlap. For matrices in file   *
C *               13, see subroutine BSMAT.                          *
C *               k points, energies and gradients will be output in *
C *               file 14. 21.08.91/mbs                              *
C *               Energy gradient calculation with distance depen-   *
C *               dent weighted Wolfsberg-Helmholz formula included. *
C *               20.05.92/mbs                                       *
C *                                                                  *
C *  Called from: CRYST                                              *
C *                                                                  *
C *  Calls......: MU                                                 *
C *                                                                  *
C ********************************************************************

      SUBROUTINE GRAD (A,D,R,DE1,B,DE2,DKSEXP,SEXP,ND,ILAST,FDIST)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION A(ND,ND),D(ND),R(ND,ND),DE1(ND),B(ND,ND),DE2(ND),
     .          DKSEXP(ND,ND),SEXP(ND,ND),DE3(NDIMMX)
      LOGICAL FDIST
      CHARACTER STRNG2*2,STRING*30

      COMMON /IOUNIT/IIN,IOUT,ISTART(40)
      COMMON /OPTV/RHO,FERMI,WSTART,WEND,DISTL,LB,LE,LBR,LER,IOVLB,
     .             IOVLE,ICHARG,NELEC,IDISTN,IEB,IEE,ISB,ISE,IHB,IHE,
     .             IPOPE,JPOPE,
     .             IREST
      COMMON /PASS/NATOM,NDIM
 
      COMMON /INDX/NCRYST,NCEL1,NCEL2,NCEL3,XK(3),WEIGHT,TWOPI,ND1,
     .             NGRAD,NFULL,CEL1,CEL2
      LOGICAL CEL1,CEL2

      COMMON /SYMANA/TRANSV(MXSYME,3),SYMOP(MXSYME,3,3),IPERMU(MXSYME,
     .               NATMMX,2),ISYMRE(MXSYME,NDIMMX),NLINOP(3),
     .               IPOINT(3,MXSYME),ILINE(3),NSYMEL,CSYMEL(MXSYME),
     .               FSYMOP(MXSYME),FSYMAN(MXSYME),FSYM
      LOGICAL FSYMOP,FSYM,FSYMAN
      CHARACTER CSYMEL*2
C
C  initialization
C
      DO 10 I=1,NDIM
        DE1(I)=0.0D0
        DE2(I)=0.0D0
        DE3(I)=0.0D0
   10 CONTINUE

      REWIND 13
      REWIND 21
      REWIND 23

      DO 30 I=1,ND
        DO 20 J=1,ND
          R(I,J)=0.0D0
          A(I,J)=0.0D0
          DKSEXP(I,J)=0.0D0
          SEXP(I,J)=0.0D0
   20   CONTINUE
   30 CONTINUE

      DO 80 I=1,NCEL1
        READ(13) B 
        DO 50 M=1,ND
          DO 40 N=1,ND
            R(M,N)=R(M,N)+B(M,N)
   40     CONTINUE
   50   CONTINUE
        IF (FDIST) THEN
          READ(13) B 
          DO 70 M=1,ND
            DO 60 N=1,ND
              DKSEXP(M,N)=DKSEXP(M,N)+B(M,N)
   60       CONTINUE
   70     CONTINUE
        ENDIF
   80 CONTINUE

      IF (NCRYST .EQ. 1) GO TO 400

      NCEL11=NCEL1+1
      DO 180 I=1,NCEL11
        DO 170 J=1,NCEL2
          READ(13) B 
          DO 100 M=1,ND
            DO 90 N=1,ND
              R(M,N)=R(M,N)+B(M,N)
   90       CONTINUE
  100     CONTINUE
          IF (FDIST) THEN
            READ(13) B 
            DO 120 M=1,ND
              DO 110 N=1,ND
                DKSEXP(M,N)=DKSEXP(M,N)+B(M,N)
  110         CONTINUE
  120       CONTINUE
          ENDIF

          READ(13) B 
          DO 140 M=1,ND
            DO 130 N=1,ND
              A(M,N)=A(M,N)+B(M,N)
  130       CONTINUE
  140     CONTINUE

          IF (FDIST) THEN
            READ(13) B 
            DO 160 M=1,ND
              DO 150 N=1,ND
                SEXP(M,N)=SEXP(M,N)+B(M,N)
  150         CONTINUE
  160       CONTINUE
          ENDIF
  170   CONTINUE
  180 CONTINUE

      DO 280 I=2,NCEL11
        DO 270 J=1,NCEL2
          READ(13) B 
          DO 200 M=1,ND
            DO 190 N=1,ND
              R(M,N)=R(M,N)+B(M,N)
  190       CONTINUE
  200     CONTINUE
          IF (FDIST) THEN
            READ(13) B 
            DO 220 M=1,ND
              DO 210 N=1,ND
                DKSEXP(M,N)=DKSEXP(M,N)+B(M,N)
  210         CONTINUE
  220       CONTINUE
          ENDIF

          READ(13) B 
          DO 240 M=1,ND
            DO 230 N=1,ND
              A(M,N)=A(M,N)+B(M,N)
  230       CONTINUE
  240     CONTINUE
          IF (FDIST) THEN
            READ(13) B 
            DO 260 M=1,ND
              DO 250 N=1,ND
                SEXP(M,N)=SEXP(M,N)+B(M,N)
  250         CONTINUE
  260       CONTINUE
          ENDIF
  270   CONTINUE
  280 CONTINUE

      IF (NCRYST .EQ. 2) GO TO 410

      NEIBR1=2*NCEL1+1
      NEIBR2=2*NCEL2+1
      DO 390 K=1,NCEL3
        DO 380 I=1,NEIBR1
          DO 370 J=1,NEIBR2
            READ(13)  
            IF (FDIST) READ(13)  
            READ(13) B 
            DO 300 M=1,ND
              DO 290 N=1,ND
                R(M,N)=R(M,N)+B(M,N)
  290         CONTINUE
  300       CONTINUE
            IF (FDIST) THEN
              READ(13) B 
              DO 320 M=1,ND
                DO 310 N=1,ND
                  DKSEXP(M,N)=DKSEXP(M,N)+B(M,N)
  310           CONTINUE
  320         CONTINUE
            ENDIF

            READ(13) B 
            DO 340 M=1,ND
              DO 330 N=1,ND
                A(M,N)=A(M,N)+B(M,N)
  330         CONTINUE
  340       CONTINUE
            IF (FDIST) THEN
              READ(13) B 
              DO 360 M=1,ND
                DO 350 N=1,ND
                  SEXP(M,N)=SEXP(M,N)+B(M,N)
  350           CONTINUE
  360         CONTINUE
            ENDIF

  370     CONTINUE
  380   CONTINUE
  390 CONTINUE

      GOTO 420
C
C one dimension system
C
  400 NSP=1
      REWIND 21
      REWIND 23
      CALL MU(DE1,A,R,B,DKSEXP,NSP,ND,D,IEB,IEE,FDIST)

      GOTO 540
C
C two dimensions system
C
  410 REWIND 21
      READ(23)  
      READ(23)  
      READ(23)  
      WRITE(23) A 
      IF (FDIST) WRITE(23) SEXP 
      NSP=1
      REWIND 23
      CALL MU(DE1,A,R,B,DKSEXP,NSP,ND,D,IEB,IEE,FDIST)
      NSP=2
      CALL MU(DE2,A,R,B,DKSEXP,NSP,ND,D,IEB,IEE,FDIST)
      GOTO 540
C
C three dimensions system, de1 and de2
C
  420 REWIND 21
      READ(23)  
      READ(23)  
      READ(23)  
      WRITE(23) A 
      IF (FDIST) WRITE(23) SEXP 
      NSP=1
      REWIND 23
      CALL MU(DE1,A,R,B,DKSEXP,NSP,ND,D,IEB,IEE,FDIST)
      NSP=2
      CALL MU(DE2,A,R,B,DKSEXP,NSP,ND,D,IEB,IEE,FDIST)
      REWIND 23
C
C three dimensions system, de1 and de2
C
      REWIND 13
      DO 430 I=1,NCEL1
        READ(13)  
        IF (FDIST) READ(13)  
  430 CONTINUE

      NCEL11=NCEL1+1
      DO 440 I=1,NCEL11
        DO 440 J=1,NCEL2
          READ(13)  
          READ(13)  
          IF (FDIST) THEN
            READ(13)  
            READ(13)  
          ENDIF
  440 CONTINUE

      DO 450 I=2,NCEL11
        DO 450 J=1,NCEL2
          READ(13)  
          READ(13)  
          IF (FDIST) THEN
            READ(13)  
            READ(13)  
          ENDIF
  450 CONTINUE
C
C     initialization
C
      DO 460 I=1,NDIM
        DO 460 J=1,NDIM
          R(J,I)=0.0D0
          DKSEXP(J,I)=0.0D0
  460 CONTINUE

      NEIBR1=2*NCEL1+1
      NEIBR2=2*NCEL2+1
      DO 530 K=1,NCEL3
        DO 520 I=1,NEIBR1
          DO 510 J=1,NEIBR2
            READ(13) B 
            DO 480 M=1,ND
              DO 470 N=1,ND
                R(M,N)=R(M,N)+B(M,N)
  470         CONTINUE
  480       CONTINUE
            IF (FDIST) THEN
              READ(13) B 
              DO 500 M=1,ND
                DO 490 N=1,ND
                  DKSEXP(M,N)=DKSEXP(M,N)+B(M,N)
  490           CONTINUE
  500         CONTINUE
              READ(13)  
              READ(13)  
            ENDIF
            READ(13)  
            READ(13)  
  510     CONTINUE
  520   CONTINUE
  530 CONTINUE

      NSP=1
      REWIND 21
      CALL MU(DE3,A,R,B,DKSEXP,NSP,ND,D,IEB,IEE,FDIST)

  540 REWIND 23
      REWIND 13
C
C Output to .EHO
C
      IF (FSYM .AND. (ILINE(1) .GT. 0)) THEN
        DO 550 I=1,3*MXSYME
          STRING(I:I)=' '
  550   CONTINUE

        WRITE(IOUT,560)
  560   FORMAT(//,5X,'Band orbital energies, derivatives (along recip.',
     .         ' lat. vec. directions), and symmetry analysis',//)

        IPOS=1
        DO 570 I=1,NSYMEL
          IF (FSYMAN(I)) THEN
            STRING(IPOS:IPOS+2)=' '//CSYMEL(I)
            IPOS=IPOS+3
          ENDIF
  570   CONTINUE
        WRITE(IOUT,580) STRING
  580   FORMAT(8X,'Band',16X,'de',8X,'de',8X,'de',13X,'Symm. ',
     .         'operations: ',A30)

        WRITE(IOUT,590)
  590   FORMAT(28X,'__',8X,'__',8X,'__',/,7X,'Index',3X,'energy',7X,
     .         'dk1',7X,'dk2',7X,'dk3',/)

        DO 640 I=1,3
          IPOS=1
          IF (NLINOP(I) .GT. 0) THEN
            DO 620 J=1,NSYMEL
              IF (FSYMAN(J)) THEN
                DO 610 K=1,NLINOP(I)
                  IF (IPOINT(I,K) .EQ. J) THEN
                    WRITE(STRNG2,600) ILINE(I)
  600               FORMAT(I2)
                    STRING(IPOS:IPOS+2)=' '//STRNG2
                    IPOS=IPOS+3
                    GOTO 620
                  ENDIF
  610           CONTINUE
                STRING(IPOS:IPOS+2)='   '
                IPOS=IPOS+3
              ENDIF
  620       CONTINUE

            WRITE(IOUT,630) STRING
  630       FORMAT(63X,'Symmetry line:   ',A30)
          ENDIF
  640   CONTINUE

        DO 670 I=IEB,IEE
          IPOS=1
          DO 650 J=1,NSYMEL
            IF (FSYMAN(J)) THEN
              WRITE(STRNG2,600) ISYMRE(J,I)
              STRING(IPOS:IPOS+2)=' '//STRNG2
              IPOS=IPOS+3
            ENDIF
  650     CONTINUE
          WRITE(IOUT,660) I,D(I),DE1(I),DE2(I),DE3(I),STRING
  660     FORMAT(5X,I5,F12.6,3F10.4,29X,A30)
  670   CONTINUE
      ELSE
        WRITE(IOUT,680)
  680   FORMAT(//,5X,'Band orbital energies and derivatives (along ',
     .         ' recip. lat. vec. directions)',//)
        WRITE(IOUT,690)
  690   FORMAT(8X,'Band',21X,'de',8X,'de',8X,'de')
        WRITE(IOUT,590)
        WRITE(IOUT,700) (I,D(I),DE1(I),DE2(I),DE3(I),I=IEB,IEE)
  700   FORMAT(6X,I5,F12.6,3F10.4)
      ENDIF

      IF (ILAST .NE. 0) THEN
        TOTALE=0.0D0
        DO 710 I=1,ILAST
          TOTALE=TOTALE+D(I)*2.0D0
  710   CONTINUE
        WRITE(IOUT,720) FERMI,ILAST,TOTALE
  720   FORMAT(//,5X,'For fermi level=',F12.6,',  ',I3,' levels are ',
     .         'doubly occupied',/,5X,'Total energy    =',F12.4,' eV',
     .         //)
      ENDIF
C
C Output to .EHE
C
      DO 730 I=1,3
        XK(I)=XK(I)/TWOPI
  730 CONTINUE

      WRITE(14,740) XK(1),XK(2),XK(3)
  740 FORMAT(3F10.7)

      DO 760 I=1,3
        WRITE(14,750) ILINE(I),NLINOP(I),(IPOINT(I,J),J=1,MXSYME)
  750   FORMAT(12I3)
  760 CONTINUE

      DO 780 I=1,LE
        WRITE(14,770) D(I),DE1(I),DE2(I),DE3(I),I,
     .                (ISYMRE(J,I),J=1,MXSYME)

  770   FORMAT(4F10.5,I4,10I3)
  780 CONTINUE

      DO 790 I=1,3
        XK(I)=XK(I)*TWOPI
  790 CONTINUE

      RETURN
      END
