C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  LOVLAP                                                          *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates the overlap component independent of    *
C *               the angle between the atoms.                       *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 16.11.92/mbs (Martin Braendle, Inst. f. *
C *               Inorg. + Phys. Chemistry, Freiestr. 3, CH-3012     *
C *               Berne)                                             *
C *  Remarks....: Problem solved with huge numbers occuring if one   *
C *               uses d- or f-elements solved. Routine doesn't need *
C *               to be compiled with the G_FLOATING (VAX/VMS) qua-  *
C *               lifier (it isn't also recommended to mix D_ and    *
C *               G_FLOATING in the same program).                   *
C *                                                                  *
C *  Called from: MOV                                                *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************
 
      SUBROUTINE LOVLAP(STRAD,A,B)
 
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
 
      DIMENSION A(30),B(30),FACT(25),BINCOE(8,8)
 
      COMMON /LOCLAP/SK1,SK2,RR,L1,L2,M,N1,N2,MAXCAL
 
      DATA BINCOE /8*1.0D0,
     .             0.0D0,1.0D0,2.0D0,3.0D0,4.0D0,5.0D0,6.0D0,7.0D0,
     .             2*0.0D0,1.0D0,3.0D0,6.0D0,10.0D0,15.0D0,21.0D0,
     .             3*0.0D0,1.0D0,4.0D0,10.0D0,20.0D0,35.0D0,
     .             4*0.0D0,1.0D0,5.0D0,15.0D0,35.0D0,
     .             5*0.0D0,1.0D0,6.0D0,21.0D0,
     .             6*0.0D0,1.0D0,7.0D0,
     .             7*0.0D0,1.0D0/
 
      FACT(1)=1.0D0
 
      DO 10 I=2,25
   10 FACT(I)=FACT(I-1)*DBLE(I-1)
 
      M2=M
      STRAD=0.0D0
      RHOA=RR*SK1
      RHOB=RR*SK2
      RHOAP=RHOA**N1
      RHOAP=RHOAP*RHOAP
      RHOAP=RHOAP*RHOA
      RHOAB=RHOB**N2
      RHOAB=RHOAB*RHOAB
      RHOAB=RHOAB*RHOB
      RHOSQR=DSQRT(RHOAP)*DSQRT(RHOAB)
      TERM2=0.5**(L1+L2+1)*DSQRT(DBLE((L1+L1+1)*(L2+L2+1))*
     .      FACT(L1-M+1)*FACT(L2-M+1)/(FACT(N1+N1+1)*FACT(N2+N2+1)*
     .      FACT(L1+M+1)*FACT(L2+M+1)))
      TERMA=TERM2*RHOSQR
      JEND=1+((L1-M)/2)
      KEND=1+((L2-M2)/2)
      IEB=M+1
 
      DO 30 J=1,JEND
        JU=J-1
        IAB=N1-L1+JU+JU+1
        ICB=L1-M-JU-JU+1
        CON1=FACT(L1+L1-JU-JU+1)/(FACT(L1-M-JU-JU+1)*FACT(JU+1)*
     .       FACT(L1-JU+1))
        DO 30 K=1,KEND
          KU=K-1
          CON12=CON1*FACT(L2+L2-KU-KU+1)/(FACT(L2-M2-KU-KU+1)*
     .          FACT(KU+1)*FACT(L2-KU+1))
          IEV=JU+KU+L2
          IF (2*(IEV/2) .NE. IEV) CON12=-CON12
          IBB=N2-L2+KU+KU+1
          IDB=L2-M2-KU-KU+1
          VALUE=0.0D0
          DO 20 I6=1,IEB
            DO 20 I5=1,IEB
              VALUE1=BINCOE(IEB,I6)*BINCOE(IEB,I5)
              IEV=I5+I6
              IF (2*(IEV/2) .NE. IEV) VALUE1=-VALUE1
              DO 20 I4=1,IDB
                VALUE1=-VALUE1
                VALUE2=BINCOE(IDB,I4)*VALUE1
                DO 20 I3=1,ICB
                  VALUE3=BINCOE(ICB,I3)*VALUE2
                  DO 20 I2=1,IBB
                    VALUE3=-VALUE3
                    VALUE4=BINCOE(IBB,I2)*VALUE3
                    DO 20 I1=1,IAB
                      TERM=VALUE4*BINCOE(IAB,I1)
                      IR=I1+I2+IEB+IEB-I6-I6-I3+IDB-I4+ICB-1
                      IP=IAB-I1+IBB-I2+IEB+IEB-I5-I5+ICB-I3+IDB-I4+1
   20     VALUE=VALUE+A(IP)*B(IR)*TERM
   30 STRAD=STRAD+VALUE*CON12
 
      STRAD=STRAD*TERMA
      RETURN
 
      END
