C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  MOV                                                             *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculates the overlap component independent of    *
C *               the angle between the atoms.                       *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 05.05.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: Slight modification of COMMON blocks.              *
C *                                                                  *
C *  Called from: MOVLAP                                             *
C *                                                                  *
C *  Calls......: ABFNS,LOVLAP                                       *
C *                                                                  *
C ********************************************************************

      SUBROUTINE MOV(SIGMA,PI,DELTA,PHI,I,J,R,NA,NB,LA,LB,NN)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION RLL(4),A(30),B(30)

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
C
C  EXPS,EXPP,EXPD,EXPD2,EXPF,EXPF2 have been combined into OEXP
C  C1,C2,C3,C4                     have been combined into CDF
C
      COMMON /ATOM2/OEXP(NSYMMX,6+2*NAEQMX),CDF(NSYMMX,6+2*NAEQMX),
     .              COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),
     .              X(NATMMX),Y(NATMMX),Z(NATMMX)

      COMMON /EQS/NAEQS(NATMMX)
      INTEGER NAEQS

      COMMON /LOCLAP/SK1,SK2,RR,L1,L2,M,N1,N2,MAXCAL

      DATA OLDSK1/-1.0D0/,
     .     OLDSK2/-1.0D0/,
     .     OLDR/-1.0D0/

      SIGMA=0.0D0
      PI=0.0D0
      DELTA=0.0D0
      PHI=0.0D0

      IA=1
      IB=1
      L1=LA
      L2=LB
      N1=NA
      N2=NB
      RR=R
      MAXCAL=NA+NB

      KEYI=ISYM(I)
      KEYJ=ISYM(J)

      ISKIPA=0
      ISKIPB=0

      IF (LA .GT. 0) THEN
        ISKIPA=2*(NAEQMX-1)
      ELSE
        DO 5 K=1,NAEQS(KEYI)
          IF (NA .EQ. NS(KEYI,K)) ISKIPA=K-1
5       CONTINUE
      ENDIF

      IF (LB .GT. 0) THEN
        ISKIPB=2*(NAEQMX-1)
      ELSE
        DO 8 K=1,NAEQS(KEYJ)
          IF (NB .EQ. NS(KEYJ,K)) ISKIPB=K-1
8       CONTINUE
      ENDIF

C
C  if double zeta orbitals used for d or f
C  sollte man auch noch verallgemeinern!!
C
      IF ( (LA .GE. 2) .OR. (LB. GE. 2) ) THEN 
        IDBL2=(LA+1)*2+ISKIPA
        IF (CDF(KEYI,IDBL2) .NE. 0.0D0) IA=2
        IDBL2=(LB+1)*2+ISKIPB
        IF (CDF(KEYJ,IDBL2) .NE. 0.0D0) IB=2
      ENDIF
C
C  loop over orbitals
C
      DO 30 IK=1,IA
        DO 30 IL=1,IB
          LC=2*LA+ISKIPA+IK
          SK1=OEXP(KEYI,LC)
          LD=2*LB+ISKIPB+IL
          SK2=OEXP(KEYJ,LD)
          IF ((OLDSK1 .EQ. SK1) .AND. (OLDSK2 .EQ. SK2) .AND. 
     .        (OLDR .EQ. RR)) GOTO 10
          CALL ABFNS(A,B)
          OLDSK1=SK1
          OLDSK2=SK2
          OLDR=RR
   10     DO 20 IJ=1,NN
            M=IJ-1
            CALL LOVLAP(RLL(IJ),A,B)
   20     CONTINUE

          XX=CDF(KEYI,LC)
          YY=CDF(KEYJ,LD)
          SIGMA=SIGMA+XX*YY*RLL(1)
          PI=PI+XX*YY*RLL(2)
          DELTA=DELTA+XX*YY*RLL(3)
          PHI=PHI+XX*YY*RLL(4)
   30 CONTINUE

      RETURN
      END
