C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  MU                                                              *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Calculation of the gradients DE1, DE2 or DE3 in    *
C *               between IE1 and IE2. S. routine GRAD.              *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 20.05.92/mbs, 18.02.93/mbs (Martin      *
C *               Braendle c/o Calzaferri group, Inst. f. Inorg. and *
C *               Phys. Chemistry, Freiestrasse 3, CH-3012 Berne)    *
C *  Remarks....: Energy gradient calculation with distance depen-   *
C *               dent weighted Wolfsberg-Helmholz formula included. *
C *               20.05.92/mbs                                       *
C *               Individual (kappa,delta). 18.02.93/mbs             *
C *                                                                  *
C *  Called from: GRAD                                               *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE MU(D,A,R,B,DKSEXP,NSP,N,ENER,IE1,IE2,FDIST)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'   
 
      DIMENSION D(N),A(N,N),R(N,N),B(N,N),DKSEXP(N,N),ENER(N)
      LOGICAL   FDIST

      COMMON /ATOMPT/IATMPT(NDIMMX)
      COMMON /DISTK/CKAPPA,CDELTA,AKAPPA(NATMMX,NATMMX),
     .              ADELTA(NATMMX,NATMMX)

      IF (NSP .EQ. 2) GOTO 10
      READ(23)  
      READ(23) A 
      READ(23) B 
      REWIND 23
      GOTO 20
   10 READ(23) R
      IF (FDIST) READ(23) DKSEXP 
      REWIND 23
   20 CONTINUE
C
C                                  T
C Until loop label 70: Calculate Cn (k)*[-en(k)*dS(k)/dk]*Cn(k).
C R holds dS(k)/dk. B holds Im(C(k)). A holds Re(C(k)).
C 20.05.92/mbs
C
      DO 60 I=IE1,IE2
        DO 50 J=1,N
          IF (J .EQ. 1) GOTO 40
          J1=J-1
          DO 30 K=1,J1
            S=(B(K,I)*A(J,I)-B(J,I)*A(K,I))*R(J,K)+(A(K,I)*A(J,I)+
     .         B(K,I)*B(J,I))*R(K,J)
            D(I)=D(I)+S+S
   30     CONTINUE
   40     D(I)=D(I)+(A(J,I)*A(J,I)+B(J,I)*B(J,I))*R(J,J)
   50   CONTINUE
   60 CONTINUE

      DO 70 I=IE1,IE2
        D(I)=-D(I)*ENER(I)
   70 CONTINUE
C
C Until loop label 90: Calculate dH(k)/dk, which will be held in R.
C Read K/2*(Htt(0,0)+Huu(0,0)) in A, which overwrites Re(C(k)).
C 20.05.92/mbs
C
      READ(22)  
      READ(22) A 
      REWIND 22
      DO 90 J=1,N
        DO 80 K=1,N
          IF (FDIST) THEN
            IATOM1=IATMPT(J)
            IATOM2=IATMPT(K)
            CON=1.0D0+AKAPPA(IATOM1,IATOM2)
            FAC=1.0D0-1.0D0/CON
            R(J,K)=A(J,K)*(R(J,K)/CON+FAC*DKSEXP(J,K))
          ELSE
            R(J,K)=R(J,K)*A(J,K)
          ENDIF 
   80   CONTINUE
   90 CONTINUE
C
C                                       T
C Read again Re(C(k)) in A. Calculate Cn (k)*[dH(k)/dk]*Cn(k).
C 20.05.92/mbs
C

      READ(23)  
      READ(23) A 
      READ(23)  
      DO 130 I=IE1,IE2
        DO 120 J=1,N
          IF (J .EQ. 1) GOTO 110
          J1=J-1
          DO 100 K=1,J1
            S=(B(K,I)*A(J,I)-B(J,I)*A(K,I))*R(J,K)+(A(K,I)*A(J,I)+
     .         B(K,I)*B(J,I))*R(K,J)
            D(I)=D(I)+S+S
  100     CONTINUE
  110     D(I)=D(I)+(A(J,I)*A(J,I)+B(J,I)*B(J,I))*R(J,J)
  120   CONTINUE
  130 CONTINUE

      RETURN
      END
