C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  PARAM                                                           *
C *                                                                  *
C *  Routine of EHMACC                                               *
C *                                                                  *
C *  Usage......: Outputs the a "MOBY" file per k-point. These files *
C *               will be used along the cEDiT program to calculate  *
C *               oscillator strength.                               * 
C *                                                                  * 
C *  Authors....: Ruedi Rytz                                         *
C *               University of Berne                                *
C *               Institute for Inorg. and Phys. Chemistry           *
C *               Freiestrasse 3, 3012 Bern, Switzerland             *
C *  Date.......: 1996                                               *
C *  Revision...:                                                    *
C *                                                                  *
C *  Remarks....:                                                    *
C *                                                                  *
C *  Called from: CRYST                                              *
C *                                                                  *
C *  Calls......: -                                                  *
C *                                                                  *
C ********************************************************************

      SUBROUTINE  OEDIT(NDIM,NCOUNT,XK,INATOM,IEB,IEE,ENERGY,X,Y,Z,
     .                  CORE,COIM,WEIGHT)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)  

      INCLUDE 'sizes'  

      INTEGER NDIM,NCOUNT,INATOM,IEB,IEE

      DIMENSION XK(3),ENERGY(NDIM),X(NATMMX),Y(NATMMX),Z(NATMMX),
     .          CORE(NDIM,NDIM),COIM(NDIM,NDIM)

      COMMON /SORBIT/CSORB(NDIMMX)
      INTEGER CSORB

      INTEGER I, J

      CHARACTER KPNAME*8, EXTENS*4
      PARAMETER(EXTENS='.kpe')
      
      PARAMETER(TWOPI=6.283185307D0)

      WRITE(KPNAME, '(I3.3,A)' ) NCOUNT, EXTENS
      OPEN (UNIT=50,FORM='formatted',STATUS='new',FILE=KPNAME)

      WRITE (50, '(A)' ) 'KPE'
      WRITE (50, '(4(1X,F11.8))' ) XK(1)/TWOPI, XK(2)/TWOPI,
     .                             XK(3)/TWOPI, WEIGHT
      WRITE (50, '(I3)' ) IEE 

      DO 10 I = IEB, IEE
         WRITE (50, '(I3,1X, E13.7E2)' ) I, ENERGY(I)
  10  CONTINUE

      WRITE (50, '(I3)' ) INATOM 
      DO 20 I = 1, INATOM
        WRITE (50, '(I3,3(1X, E13.7E2))' ) I, X(I), Y(I), Z(I)
  20  CONTINUE

       WRITE (50, '(A)' ) 'EHTB-Rechnung'
       WRITE (50, '(2I3)' ) INATOM, CSORB(INATOM+1)-1

       DO 30 I = 1, INATOM 
         WRITE (50, '(2I3)' ) I, I
         WRITE (50, '(2I3)' ) CSORB(I), CSORB(I+1)-1
  30   CONTINUE

       DO 40 I = IEB, IEE

         WRITE (50, '(2X, F8.4, I3)' ) ENERGY(I), I
         WRITE (50, '(10(1X,F7.4))' ) ( CORE(J,I), J=1,IEE )
         IF ( ABS(MOD(J-1,10)) .LT. 1.0D-5 ) THEN
            WRITE (50, '(A)' )
         ENDIF
         WRITE (50, '(10(1X,F7.4))' ) ( COIM(J,I), J=1,IEE )
         IF ( ABS(MOD(J,10)) .LT. 1.0D-5 ) THEN
            WRITE (50, '(A)' )
         ENDIF

  40   CONTINUE

      CLOSE (50,STATUS='keep')

      RETURN
      END
