C ********************************************************************
C * EHMACC - Extended Hueckel Molecular and Crystal Calculations     *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SUBROUTINE REPUL                                                *
C *                                                                  *
C *  Routine of EHMACC, QCPE # 571                                   *
C *                                                                  *
C *  Usage......: Calculates and returns core-core-repulsion energy  *
C *               EREP.                                              *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs) c/o                          *
C *               Calzaferri group c/o                               *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 11.06.92                                           *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: CRYST,HUCKEL                                       *
C *                                                                  *
C *  Calls......: REPUL1                                             *
C *                                                                  *
C ********************************************************************

      SUBROUTINE REPUL(EREP)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      COMMON /ATOM1/NS(NSYMMX,NAEQMX),NP(NSYMMX),ND(NSYMMX),NF(NSYMMX),
     .              INATOM,ISYM(NATMMX),NELEM
      COMMON /ATOM2/EXPS(NSYMMX,NAEQMX),EXPS2(NSYMMX,NAEQMX),
     .              EXPP(NSYMMX),EXPP2(NSYMMX),
     .              EXPD(NSYMMX),EXPD2(NSYMMX),
     .              EXPF(NSYMMX),EXPF2(NSYMMX),
     .              CS1(NSYMMX,NAEQMX),CS2(NSYMMX,NAEQMX),
     .              CP1(NSYMMX),CP2(NSYMMX),CD1(NSYMMX),CD2(NSYMMX),
     .              CF1(NSYMMX),CF2(NSYMMX),COULS(NSYMMX,NAEQMX),
     .              COULP(NSYMMX),COULD(NSYMMX),COULF(NSYMMX),
     .              X(NATMMX),Y(NATMMX),Z(NATMMX)
      COMMON /CELL/DIST(3,3)
      COMMON /DEX/ADD(3),NB,NE,ISUB,NSUB
      COMMON /INDX/NCRYST,NCEL1,NCEL2,NCEL3,XK(3),WEIGHT,TWOPI,ND1,
     .             NGRAD,NFULL,CEL1,CEL2
      LOGICAL CEL1,CEL2

      COMMON /AVELEC/ZCORE(NATMMX),USECOR(NATMMX)
      LOGICAL USECOR

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      PARAMETER (ECONV=27.211608D0*0.52917706D0)

      EREP=0.0D0
C
C Calculate EREP in the unit cell.
C 
      DO 20 I=2,INATOM
        KEYI=ISYM(I)
        DO 10 J=1,I-1
          KEYJ=ISYM(J)
          DELX=X(I)-X(J)
          DELY=Y(I)-Y(J)
          DELZ=Z(I)-Z(J)
          R=DSQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)
          ZA=ZCORE(I)
          ZB=ZCORE(J)
          EREP1=ZA*ZB/R-(ZA*REPUL1(R,J,KEYJ)+ZB*REPUL1(R,I,KEYI))/2.0D0
          EREP=EREP+EREP1
   10   CONTINUE
   20 CONTINUE

      IF (INDEX(KEYWRD,'BAND') .GT. 0) THEN
C
C 1D system
C
        DO 60 I1=1,NCEL1
          RI1=DBLE(I1)
          DO 30 L=1,3
            ADD(L)=RI1*DIST(1,L)
   30     CONTINUE
          DO 50 I=1,INATOM
            KEYI=ISYM(I)
            DO 40 J=1,INATOM
              KEYJ=ISYM(J)
              DELX=X(I)-X(J)+ADD(1)
              DELY=Y(I)-Y(J)+ADD(2)
              DELZ=Z(I)-Z(J)+ADD(3)
              R=DSQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)
              ZA=ZCORE(I)
              ZB=ZCORE(J)
              EREP1=ZA*ZB/R-(ZA*REPUL1(R,J,KEYJ)+ZB*REPUL1(R,I,KEYI))
     .              /2.0D0
              EREP=EREP+EREP1
   40       CONTINUE
   50     CONTINUE
   60   CONTINUE
C
C 2D system
C
        IF (NCRYST .GT. 1) THEN
          DO 110 I1=1,NCEL1+1
            RI1=DBLE(I1-1)
            DO 100 I2=1,NCEL2
              RI2=DBLE(I2)
              DO 70 L=1,3
                ADD(L)=RI1*DIST(1,L)+RI2*DIST(2,L)
   70         CONTINUE
              DO 90 I=1,INATOM
                KEYI=ISYM(I)
                DO 80 J=1,INATOM     
                  KEYJ=ISYM(J)
                  DELX=X(I)-X(J)+ADD(1)
                  DELY=Y(I)-Y(J)+ADD(2)
                  DELZ=Z(I)-Z(J)+ADD(3)
                  R=DSQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)
                  ZA=ZCORE(I)
                  ZB=ZCORE(J)
                  EREP1=ZA*ZB/R-(ZA*REPUL1(R,J,KEYJ)+
     .                  ZB*REPUL1(R,I,KEYI))/2.0D0
                  EREP=EREP+EREP1
   80           CONTINUE
   90         CONTINUE
  100       CONTINUE
  110     CONTINUE

          DO 160 I1=1,NCEL1+1
            RI1=DBLE(I1-1)
            DO 150 I2=1,NCEL2
              RI2=-DBLE(I2)
              DO 120 L=1,3
                ADD(L)=RI1*DIST(1,L)+RI2*DIST(2,L)
  120         CONTINUE
              DO 140 I=1,INATOM
                KEYI=ISYM(I)
                DO 130 J=1,INATOM    
                  KEYJ=ISYM(J)
                  DELX=X(I)-X(J)+ADD(1)
                  DELY=Y(I)-Y(J)+ADD(2)
                  DELZ=Z(I)-Z(J)+ADD(3)
                  R=DSQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)
                  ZA=ZCORE(I)
                  ZB=ZCORE(J)
                  EREP1=ZA*ZB/R-(ZA*REPUL1(R,J,KEYJ)+
     .                  ZB*REPUL1(R,I,KEYI))/2.0D0
                  EREP=EREP+EREP1
  130           CONTINUE
  140         CONTINUE
  150       CONTINUE
  160     CONTINUE
        ENDIF
C
C 3D system
C
        IF (NCRYST .GT. 2) THEN
          DO 220 I3=1,NCEL3
            RI3=DBLE(I3)
            N1=-NCEL1-1
            DO 210 I1=1,2*NCEL1+1
              N1=N1+1
              RI1=DBLE(N1)
              N2=-NCEL2-1
              DO 200 I2=1,2*NCEL2+1
                N2=N2+1
                RI2=DBLE(N2)
                DO 170 L=1,3
                  ADD(L)=RI1*DIST(1,L)+RI2*DIST(2,L)+RI3*DIST(3,L)
  170           CONTINUE
                DO 190 I=1,INATOM
                  KEYI=ISYM(I)
                  DO 180 J=1,INATOM      
                    KEYJ=ISYM(J)
                    DELX=X(I)-X(J)+ADD(1)
                    DELY=Y(I)-Y(J)+ADD(2)
                    DELZ=Z(I)-Z(J)+ADD(3)
                    R=DSQRT(DELX*DELX+DELY*DELY+DELZ*DELZ)
                    ZA=ZCORE(I)
                    ZB=ZCORE(J)
                    EREP1=ZA*ZB/R-(ZA*REPUL1(R,J,KEYJ)+
     .                    ZB*REPUL1(R,I,KEYI))/2.0D0
                    EREP=EREP+EREP1
  180             CONTINUE
  190           CONTINUE
  200         CONTINUE
  210       CONTINUE
  220     CONTINUE
        ENDIF
      ENDIF

      EREP=EREP*ECONV
      RETURN
      END
