C ********************************************************************
C * EHPC - Extended Hueckel properties calculations                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  PARRAY                                                          *
C *                                                                  *
C *  Routine of EHPC                                                 *
C *                                                                  *
C *  Usage......: Defines a single one-dimensional array a to be di- *
C *               vided into several two- and one-dimensional arrays,*
C *               which are used later. Calculation of size of array *
C *               A see parameter statement.                         *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 12.06.92/mbs (Martin Braendle, c/o      *
C *               Calzaferri group, Inst. f. Inorg. and Phys. Chem., *
C *               Freiestr. 3, CH-3012 Berne)                        *
C *  Remarks....: -                                                  *
C *                                                                  *
C *  Called from: EHPC                                               *
C *                                                                  *
C *  Calls......: PPART                                              *
C *                                                                  *
C ********************************************************************

      SUBROUTINE PARRAY

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      PARAMETER(ISIZE=NDIMMX*NDIMMX+25*NDIMMX*NKPMX+111*NDIMMX+4*MXMESH)

      DIMENSION A(ISIZE)

      COMMON /ATOM1/NSS(NSYMMX),NPP(NSYMMX),NDD(NSYMMX),NFF(NSYMMX),
     .              IATOM,ISYM(NATMMX),NELEM
      COMMON /OPTV/DELTA,EMIN,EMAX,STEP,XLIMIT,LEVB,LEVE,NMESH,IVARYS,
     .             IVARYE,NVARY
      COMMON /ORB2/ NATOM,IND(NDIMMX),NORBEF(NATMMX),NORBE(NATMMX),NDEL

      COMMON /ATO/AC(NATMMX),SYMBOL(NSYMMX)
      CHARACTER AC*7,SYMBOL*2

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /ORB1/INDT(NDIMMX,2)
      CHARACTER INDT*10

      REWIND 1
      READ (1,10) NKP,NDIM,NATOM,NELEM,NDEL5,NVARY
10    FORMAT(6I6)
      DO 30 I=1,NDIM
        READ(1,20) (INDT(I,J),J=1,2),IND(I)
20      FORMAT(2A11,I6)
30    CONTINUE
      DO 50 I=1,NELEM
        READ(1,40) SYMBOL(I),NSS(I),NPP(I),NDD(I),NFF(I)
40      FORMAT(A3,4I2)
50    CONTINUE
      DO 70 I=1,NATOM
        READ(1,60) AC(I),NORBEF(I),NORBE(I)
60      FORMAT(A8,2I6)
70    CONTINUE

      IATOM=NATOM
      NDK=NDIM*NKP
      NDIMSQ=NDIM*NDIM
      NDIM2=NDIM*2
      IF (LEVB .EQ. 0) LEVB=1
      IF (LEVE .EQ. 0) LEVE=NDIM2
      IF (LEVE .GT. NDIM2) LEVE=NDIM2
      IF (LEVB .GT. LEVE) LEVB=LEVE

      IF (INDEX(KEYWRD,'INTEGR') .GT. 0) THEN
        IF (STEP .LT. PSTEP) STEP=PSTEP
        NELEC=LEVE-LEVB
        NSTEP=INT((DBLE(NELEC)+0.01D0)/STEP)+1
      ENDIF

      IF (NSTEP .EQ. 0) NSTEP=1
      IF (NMESH .EQ. 0) NMESH=2
      NSKP=NSTEP*NKP
      J1=1
      J2=J1+NDK
      J3=J2+NDK
      J4=J3+NDK
      J5=J4+NDIM
      J6=J5
      J7=J6+NDIMSQ
      J8=J7+NSKP
      J9=J8+NSTEP
      J10=J9+NSTEP
      J11=J10+NSTEP
      J12=J11+NSTEP
      J13=J12+NSTEP
      J14=J7
      J15=J14+NMESH
      J16=J15+NMESH
      J17=J16+NMESH
      ITOTC1=J5+NDK
      ITOTC2=J13+NSTEP
      ITOTC3=J17+NMESH
      ITOTC=MAX(ITOTC1,ITOTC2,ITOTC3)
      IF (ITOTC .GT. ISIZE) THEN
        SLINE='Dimension too large, modify EHPPAR parameters.'
        CALL WRITEL(SLINE)
        CALL CLEXIT
      ENDIF

      CALL PPART(A(J1),A(J2),A(J3),A(J4),A(J5),A(J6),A(J7),A(J8),A(J9),
     .           A(J10),A(J11),A(J12),A(J13),A(J14),A(J15),A(J16),
     .           A(J17),NDIM,NKP,NDK,NSTEP)

      RETURN
      END
