C ********************************************************************
C * EHPC - Extended Hueckel properties calculations                  *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SMOOTB                                                          *
C *                                                                  *
C *  Routine of EHPC                                                 *
C *                                                                  *
C *  Usage......: Applies gaussian smoothing to generate crystal     *
C *               orbital overlap population (COOP) as a function of *
C *               energy (for a set of given mesh points) for plot-  *
C *               ting purposes.                                     *
C *                                                                  *
C *  Authors....: M.-H. Whangbo, Dept of Chem, North Carolina State  *
C *               University, Raleigh N.C.,                          *
C *               modified by M. Evain, Dept of Chemistry, North     *
C *               Carolina State University, Raleigh N.C.            *
C *  Date.......: 1981                                               *
C *  Revision...: Dec. 1987, 31.03.92/mbs, 9.04.92/mbs, 12.6.92/mbs, *
C *               14.10.93/mbs 21.9.94/mbs                           *
C *               (Martin Braendle c/o Calzaferri group, Institute   *
C *               for Inorganic and Physical Chemistry, Freiestr. 3, *
C *               CH-3012 Berne)                                     *
C *                                                                  *
C *  Remarks....: Formatted output will be read and generated.       *
C *               Slight modification of COMMON blocks.              *
C *               Smoothed integrated COOP implemented.              *
C *               Geometry variations implemented.                   *
C *                                                                  *
C *  Called from: PPART                                              *
C *                                                                  *
C *  Calls......: WRITEL,OUTPUT                                      *
C *                                                                  *
C ********************************************************************

      SUBROUTINE SMOOTB(E,EDX,XND,XNDI,XNORM,NDIM,NKP,NDK)

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      INCLUDE 'sizes'

      DIMENSION    E(NDIM),EDX(NMESH),XND(NMESH),XNDI(NMESH),XK(3)
      CHARACTER*80 CDUMMY
      CHARACTER*7  ACI1,ACI2,ACJ1,ACJ2
      LOGICAL      T1,T2,T3,T4,T5

      COMMON /ATOM1/NSS(NSYMMX),NPP(NSYMMX),NDD(NSYMMX),NFF(NSYMMX),
     .              IATOM,ISYM(NATMMX),NELEM

      COMMON /BOND1/IDAT(NBONMX,2)
      CHARACTER*7 IDAT

      COMMON /BOND2/DDX(NBONMX),NDIVIS(NBONMX),NEQU
      COMMON /ENERG/IENERG
      COMMON /IOUNIT/IIN,IOUT,ISTART(40)

      COMMON /KEYWRD/KEYWRD,SLINE
      CHARACTER KEYWRD*320,SLINE*80

      COMMON /OPTV/DELTA,EMIN,EMAX,STEP,XLIMIT,LEVB,LEVE,NMESH,IVARYS,
     .             IVARYE,NVARY
      COMMON /OVLP/IPROJB(NBONMX,MXCARD),NDPRB(MXCARD),NPROJB,
     .             NBTOT(MXCARD)
      COMMON /OVLPRO/DTMIN(NBONMX,MXCARD),DTMAX(NBONMX,MXCARD),
     .               IDMAX(MXCARD),NOM1(NBONMX,MXCARD),
     .               NOM2(NBONMX,MXCARD)
      CHARACTER*7 NOM1,NOM2

      COMMON /POP/DD(NBONMX,NDIMMX)

      IAV=INDEX(KEYWRD,'AVERAGE')

      IF (DELTA .EQ. 0.0D0) DELTA=PDELTA
      IF (XLIMIT .EQ. 0.0D0) XLIMIT=PLIMIT
      ELIM=-LOG(XLIMIT)
      ELIM=SQRT(ELIM)*DELTA

      XSTEP=ABS(EMAX-EMIN)/DBLE(NMESH-1)
      DO 10 I=1,NMESH
        EDX(I)=EMIN+XSTEP*DBLE(I-1)
   10 CONTINUE

      REWIND 1
      REWIND 3
      REWIND 8

      DO 30 I=1,NDIM+NELEM+IATOM+1
        READ(1,20) CDUMMY
   20   FORMAT(A80)
   30 CONTINUE
      IF (IVARYS .GT. 1) THEN
        DO 110 IVARCT=1,IVARYS-1
          DO 40 IKK=1,NKP*(NDIM+1)+NDK+1
            READ(1,20) CDUMMY
   40     CONTINUE
          READ(3,50) NEQU
   50     FORMAT(I4)
          DO 100 IKK=1,NKP
            READ(3,60) IOVLB,IOVLE
   60       FORMAT(2I4)
            ILOOP=IOVLE-IOVLB+2
            IF (IOVLB .EQ. 1) ILOOP=ILOOP-1
            IF (IKK .EQ. 1) THEN
              DO 70 IB=1,NEQU
                READ(3,20) CDUMMY
   70         CONTINUE
            ENDIF
            DO 90 IL=1,ILOOP
              READ(3,80) (DD(IDD,IL),IDD=1,NEQU)
C
C old version files (prior to 29.06.93)/mbs
C   80         FORMAT(F10.5)
C
   80         FORMAT(F8.5)
   90       CONTINUE
  100     CONTINUE
  110   CONTINUE
      ENDIF

      READ(3,50) NEQU
      DO 180 IKK=1,NKP
        READ(1,120) (XK(I),I=1,3),WEI
  120   FORMAT(4F12.6)
        WRITE(8,130) WEI
  130   FORMAT(F12.6)
        DO 140 I=1,NDIM
          READ(1,130) E(I)
          WRITE(8,130) E(I)
  140   CONTINUE
        READ(3,60) IOVLB, IOVLE
        WRITE(8,60) IOVLB, IOVLE
        ILOOP=IOVLE-IOVLB+2
        IF (IOVLB .EQ. 1) ILOOP=ILOOP-1
        IF (IKK .EQ. 1) THEN
          DO 160 LOQ=1,NEQU
            READ(3,150) IDAT(LOQ,1),IDAT(LOQ,2),DDX(LOQ),NDIVIS(LOQ)
  150       FORMAT(2A7,F10.4,I4)
  160     CONTINUE
        ENDIF
        DO 170 IL=1,ILOOP
          READ(3,80) (DD(IDD,IL),IDD=1,NEQU)
          WRITE(8,80) (DD(IDD,IL),IDD=1,NEQU)
  170   CONTINUE
  180 CONTINUE

      DO 570 IPRO=1,NPROJB
C
C Initialization
C
        REWIND 8
        DO 190 I=1,NMESH
          XND(I)=0.0D0
          XNDI(I)=0.0D0
  190   CONTINUE
C
        IF (NBTOT(IPRO) .EQ. 0) THEN
C
C Search of bonds has not yet been performed.
C
          IC=0
          DO 210 I=1,NEQU
            ACI1=IDAT(I,1)
            ACI2=IDAT(I,2)
            DIS=DDX(I)
            DO 200 J=1,IDMAX(IPRO)
              ACJ1=NOM1(J,IPRO)
              ACJ2=NOM2(J,IPRO)
              DMI=DTMIN(J,IPRO)
              DMA=DTMAX(J,IPRO)
              T1=(ACI1 .EQ. ACJ1)
              T2=(ACI2 .EQ. ACJ2)
              T3=(ACI1 .EQ. ACJ2)
              T4=(ACI2 .EQ. ACJ1)
              T1=(T1 .AND. T2 .OR. T3 .AND. T4)
              T5=((DIS .GT. DMI) .AND. (DIS .LT. DMA))
              IF ( .NOT. (T1 .AND. T5)) GOTO 200
              IC=IC+1
              IPROJB(IC,IPRO)=I
              GOTO 210
  200       CONTINUE
  210     CONTINUE
          NDPRB(IPRO)=IC

          IF (IC .NE. 0) THEN
            NBTOT(IPRO)=0
            WRITE(IOUT,220) IC
  220       FORMAT(/////,' Overlap population: ',I4,
     .             ' bond type(s) projected',/)
            DO 240 I=1,IC
              J=IPROJB(I,IPRO)
              NBTOT(IPRO)=NBTOT(IPRO)+NDIVIS(J)
              WRITE(IOUT,230) IDAT(J,1),IDAT(J,2),DDX(J),NDIVIS(J)
  230         FORMAT(4X,A7,' <---> ',A7,'  with the distance ',F10.5,
     .               ' A, ',I2,' equivalent bond(s)')
  240       CONTINUE
            WRITE(IOUT,250) NBTOT(IPRO)
  250       FORMAT(/,'    Total of ',I3,' bond(s).',/)
          ELSE
            SLINE='no bond'
            CALL WRITEL (SLINE)
          ENDIF
        ENDIF

        XBE=0.0D0
        DO 480 IKK=1,NKP
          READ(8,130) WEI
          DO 260 I=1,NDIM
            READ(8,130) E(I)
  260     CONTINUE
          READ(8,60) IOVLB, IOVLE
          ICUMUL=IOVLB-1
          ILOOP=IOVLE-IOVLB+2
          IF (IOVLB .EQ. 1) ILOOP=ILOOP-1
          DO 270 IL=1,ILOOP
            READ(8,80) (DD(IB,IL),IB=1,NEQU)
  270     CONTINUE
          WEI=WEI*XNORM
C
C Look for the last point
C
          IRUN=NMESH
          N3=NDIM
          N1=1
  280     N2=(N3+N1)/2
          IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 290
          IF (E(N2) .GT. EDX(IRUN)) THEN
            N3=N2
          ELSE
            N1=N2
          ENDIF
          GOTO 280
  290     NEND=N2
          IF (E(N2) .GT. EDX(IRUN)) NEND=NEND-1
C
C Look for the first point
C
          IRUN=1
          N3=NDIM
          N1=1
  300     N2=(N3+N1)/2
          IF ((N1 .GE. N2) .OR. (N3 .LE. N2)) GOTO 310
          IF (E(N2) .GT. EDX(IRUN)) THEN
            N3=N2
          ELSE
            N1=N2
          ENDIF
          GOTO 300
  310     NSTART=N2
          IF (E(N2) .LT. EDX(IRUN)) NSTART=NSTART+1
          NST=NSTART
C
C XNADDB: Tail area of COOP gaussian below EMIN
C
          XNADDB=0.0D0
C
C Fit gaussian slopes onto energy mesh.
C IRUN runs through all energy mesh points.
C
          IRUN=0
  320     IRUN=IRUN+1
          IF (IRUN .GT. NMESH) GOTO 410
          CVAL=EDX(IRUN)
          IF (IRUN .GT. 1) THEN
            DO 330 IP=NST,NDIM
              IF (E(IP) .GE. CVAL) GOTO 340
              NST=NST+1
  330       CONTINUE
          ENDIF
  340     NCOU=NST
          IF (NCOU .GT. NDIM) GOTO 370
C
C Count levels NCOU until the height of the gaussian slope lies below
C LIMIT, which is expressed by the energy difference DIF larger than
C ELIM. Right k-space part of energy level CVAL.
C
  350     DIF=ABS(E(NCOU)-CVAL)
          IF (DIF .GT. ELIM) GOTO 370
          IF (NCOU .GT. IOVLE) THEN
            SLINE='The smoothing requires levels above the limitation'
     .            //' OVLPOP=m to n of EHMACC.'
            CALL WRITEL(SLINE)
            CALL CLEXIT
          ENDIF
          Z=DIF/DELTA
          Z=-Z*Z
          Z=DEXP(Z)*WEI
          MM=NCOU-ICUMUL+1
          DO 360 N=1,NDPRB(IPRO)
            NN=IPROJB(N,IPRO)
            XND(IRUN)=XND(IRUN)+DD(NN,MM)*Z
  360     CONTINUE
          NCOU=NCOU+1
          IF (NCOU .LE. IOVLE) GOTO 350
C
C Count levels NCOU until the height of the gaussian slope lies below
C LIMIT, which is expressed by the energy difference DIF larger than
C ELIM. Left k-space part of energy level CVAL.
C
  370     NCOU=NST-1
  380     IF (NCOU .LE. 0) GOTO 400
          DIF=ABS(E(NCOU)-CVAL)
          IF (DIF .GT. ELIM) GOTO 400
          IF (NCOU .LT. (IOVLB-1)) THEN
            SLINE='The smoothing requires levels below the limitation'
     .            //' OVLPOP=m to n of EHMACC.'
            CALL WRITEL(SLINE)
            CALL CLEXIT
          ENDIF
          Z=DIF/DELTA
          Z=-Z*Z
          Z=DEXP(Z)*WEI
          MM=NCOU-ICUMUL+1
          DO 390 N=1,NDPRB(IPRO)
            NN=IPROJB(N,IPRO)
            XND(IRUN)=XND(IRUN)+DD(NN,MM)*Z
            IF (NCOU .LT. NSTART) THEN
              XNADDB=XNADDB+DD(NN,MM)*Z
            ENDIF
  390     CONTINUE
          NCOU=NCOU-1
          GOTO 380
  400     GOTO 320
  410     CONTINUE
C
C XNLTB: COOP below EMIN
C
          XNLTB=0.0D0
C
C Determine COOP below EMIN by the above procedure.
C
          NCOU=-1
          XRUN=EDX(1)
  420     XRUN=XRUN-XSTEP
          IF (NCOU .EQ. NSTART) GOTO 450
          NCOU=NSTART
  430     DIF=ABS(E(NCOU)-XRUN)
          IF (DIF .GT. ELIM) GOTO 420
          Z=DIF/DELTA
          Z=-Z*Z
          Z=DEXP(Z)*WEI
          MM=NCOU-ICUMUL+1
          DO 440 N=1,NDPRB(IPRO)
            NN=IPROJB(N,IPRO)
            XNLTB=XNLTB+DD(NN,MM)*Z
  440     CONTINUE
          NCOU=NCOU+1
          IF (NCOU .GT. IOVLE) GOTO 420
          GOTO 430
  450     CONTINUE
          DO 470 I=ICUMUL,NSTART-1
            MM=I-ICUMUL+1
            DO 460 N=1,NDPRB(IPRO)
              NN=IPROJB(N,IPRO)
              XBE=XBE+WEI/XNORM*DD(NN,MM)
  460       CONTINUE
  470     CONTINUE
  480   CONTINUE
        XBE=XBE+(XNLTB-XNADDB)*XSTEP
C
C Sum up COOP.
C
        DO 490 I=1,NMESH
          XBE=XBE+XSTEP*XND(I)
          XNDI(I)=XBE
  490   CONTINUE
C
C Output energy mesh
C
        IF (IENERG .EQ. 0) THEN
          WRITE(IOUT,500)
  500     FORMAT(//////,5X,'Energy mesh points',//)
          CALL OUTPUT(EDX,NMESH)
          IENERG=1
          WRITE(7,510) EDX
  510     FORMAT(F12.5)
        ENDIF
C
C Output of OVLPOP
C
        IF (IAV .EQ. 1) THEN
          DO 520 II=1,NMESH
            XND(II)=XND(II)/NBTOT(IPRO)
            XNDI(II)=XNDI(II)/NBTOT(IPRO)
  520     CONTINUE
        ENDIF

        IF (IAV .EQ. 1) THEN
          WRITE(IOUT,530) IPRO
  530     FORMAT(//////,5X,'Averaged smoothed overlap population for ',
     .           'the projection #',I2,//)
        ELSE
          WRITE(IOUT,540) IPRO
  540     FORMAT(//////,5X,'Cumulated smoothed overlap population for ',
     .           'the projection #',I2,//)
        ENDIF
        CALL OUTPUT(XND,NMESH)
        WRITE(7,510) XND

        IF (IAV .EQ. 1) THEN
          WRITE(IOUT,550) IPRO
  550     FORMAT(//////,5X,'Averaged smoothed integrated overlap ',
     .           'population for the projection #',I2,//)
        ELSE
          WRITE(IOUT,560) IPRO
  560     FORMAT(//////,5X,'Cumulated smoothed integrated overlap ',
     .           'population for the projection #',I2,//)
        ENDIF
        CALL OUTPUT(XNDI,NMESH)
        WRITE(7,510) XNDI
  570 CONTINUE
      RETURN
      END
