C ********************************************************************
C * KMESHPEG - Program Package in Order to Compute k-sets            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  HEXAG1                                                          *
C *                                                                  *
C *  Usage......: Generates k-point mesh of hexagonal lattice, grid  *
C *               type 1a, according to R. Ramirez, M.C. Boehm,      *
C *               Int. J. Quant. Chem. 30 (1986) 391                 *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group c/o                               *
C *               Prof. Dr. Gion Calzaferri                          *
C *               Institute for anorg. + phys. Chemistry             *
C *               University of Berne                                *
C *               Freiestrasse 3                                     *
C *               CH-3012 Berne                                      *
C *                                                                  *
C *  Date.......: 15.06.93/mbs                                       *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM HEXAG1

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      CHARACTER*80 STRING

      WRITE(6,10)
10    FORMAT(/,' Generate k-point mesh (hexagonal lattice, type 1a)',/)
20    WRITE(6,30)
30    FORMAT(' Number of k-points along x direction, total is ',
     .       ' (N*N+N+1)/3: ',$)
      READ(5,40,ERR=20) N
40    FORMAT(I3)
      IF (N .LT. 4) GOTO 20
      IF (INT(DBLE(N-4)/3.0D0) .NE. (N-4)/3) GOTO 20

      WRITE(6,50)
50    FORMAT(' Enter optional string for output options: ',$)
      READ(5,60) STRING
60    FORMAT(A80)

      DX=0.5D0/DBLE(N-1)
      NKMAX=(N*N+N+1)/3
      ONETHD=1.0D0/3.0D0
      DELTA=1.0D-9

      ICOUNT=0
      X=0.0D0
      Y=0.0D0

      WRITE(1,65) NKMAX
65    FORMAT('!',/,
     .       '! Hexagonal lattice, grid type 1a, ',I3,' k-points,',/,
     .       '! according to R. Ramirez, M.C. Boehm, Int. J. Quant. '
     .       'Chem. 30 (1986), 391',/,'!')

70    IF (X .EQ. Y) THEN
        IF (X .EQ. 0.0D0) THEN
          IWEIGT=1
        ELSE IF (X .LT. ONETHD) THEN
          IWEIGT=6
        ELSE 
          IWEIGT=2
        ENDIF
      ELSE
        IF (Y .EQ. 0.0D0) THEN
          IF (X .EQ. 0.5D0) THEN
            IWEIGT=3
          ELSE
            IWEIGT=6
          ENDIF
        ELSE IF (Y .EQ. 1.0D0-2.0D0*X) THEN
          IWEIGT=6
        ELSE
          IWEIGT=12
        ENDIF
      ENDIF
      WRITE(1,80) X,Y,0.0D0,IWEIGT
80    FORMAT('KPIN ',3F13.10,1X,I2)
      IF (STRING(1:1) .GT. ' ') THEN
        WRITE(1,90) STRING
90      FORMAT(A80)
      ENDIF
      Y=Y+DX
      IF ((Y .GT. X) .OR. (Y .GT. 1.0D0-2*X+DELTA)) THEN
        X=X+DX
        Y=0.0D0
      ENDIF
      IF (X .LE. 0.5D0) GOTO 70

      STOP
      END
