C ********************************************************************
C * KMESHPEG - Program Package in Order to Compute k-sets            *
C *                                                                  *
C * Copyright (C) 1997 by Gion Calzaferri                            *
C *                                                                  *
C * This program is free software; you can redistribute it and/or    *
C * modify it under the terms of the GNU General Public License as   *
C * published by the Free Software Foundation; Version 2 of the      *
C * License.                                                         *
C *                                                                  *
C * This program is distributed in the hope that it will be useful,  *
C * but WITHOUT ANY WARRANTY; without even the implied warranty of   *
C * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU *
C * General Public License for more details.                         *
C *                                                                  *
C * You should have received a copy of the GNU General Public        *
C * License along with this program; if not, write to the Free       *
C * Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,   *
C * MA 02111-1307 USA                                                *
C ********************************************************************
C ********************************************************************
C *                                                                  *
C *  SC                                                              *
C *                                                                  *
C *  Usage......: Generates special k point set for the simple cubic *
C *               lattice (similar to a method described by A.H.     *
C *               McDonald, Phys. Rev. B 18(1978), 5897 for the fcc  *
C *               and bcc lattices).                                 *
C *                                                                  *
C *  Author.....: Martin Braendle (mbs)                              *
C *  Available..: Calzaferri group, Inst. f. Inorganic and Physical  *
C *               Chemistry, University of Berne, Freiestr. 3,       *
C *               CH-3012 Berne                                      *
C *  Date.......: 29.06.93/mbs                                       *
C *  Revision...: -                                                  *
C *  Remarks....: -                                                  *
C *                                                                  *
C ********************************************************************

      PROGRAM SC

      IMPLICIT DOUBLE PRECISION(A-H,O-Z)

      CHARACTER*80 STRING

      WRITE(6,10)
10    FORMAT(' k point set for the simple cubic lattice.',/)
20    WRITE(6,30)
30    FORMAT(' Enter q (number of k points on DELTA line): ',$)
      READ(5,40,ERR=20) IQ
40    FORMAT(I3)
      IF (IQ .LT. 2) GOTO 20

      NQ=IQ+1
      N=(NQ**3-NQ)/6

      WRITE(6,50) N
50    FORMAT(' Approximate number of k points: ',I4)

      WRITE(6,60)
60    FORMAT(' Enter optional string for output options: ',$)
      READ(5,70) STRING
70    FORMAT(A80)

      WRITE(1,75) N
75    FORMAT('!',/,'! sc lattice, approx. ',I3,' k-points,',/,
     .       '!  similar to method described by A.H. McDonald, ',/,
     .       'Phys. Rev. B 18(1978)',/,'!')

      N=0
      IQ=IQ-1
      Q=2.0D0*DBLE(IQ)

      DO 120 NZ=0,IQ
        DO 110 NY=0,NZ
          DO 100 NX=0,NY
C
C Determine weights of k points.
C
C General point.
C
            NWEIGT=48
            IF ((NX .EQ. NY) .AND. (NY .EQ. NZ)) THEN
C
C LAMBDA axis.
C
              NWEIGT=8
              IF ((NX .EQ. 0) .OR. (NX .EQ. IQ)) THEN
C
C GAMMA or R point.
C
                NWEIGT=1
              ENDIF
            ELSE IF (NX .EQ. 0) THEN
C
C GAMMA-M-X plane.
C
              NWEIGT=24
              IF (NY .EQ. NZ) THEN
C
C SIGMA axis.
C
                NWEIGT=12
                IF (NY .EQ. IQ) THEN
C
C M point.
C
                  NWEIGT=4
                ENDIF
              ELSE IF (NY .EQ. 0) THEN
C
C DELTA axis.
C
                NWEIGT=6
                IF (NZ .EQ. IQ) THEN
C
C X point.
C
                  NWEIGT=4
                ENDIF
              ELSE IF (NZ .EQ. IQ) THEN
C
C Z axis.
C
                NWEIGT=12
              ELSE
              ENDIF
            ELSE IF (NZ .EQ. IQ) THEN
C
C M-R-X plane.
C
              NWEIGT=24
              IF (NX .EQ. NY) THEN
C
C S axis.
C
                NWEIGT=12
              ELSE IF (NY .EQ. IQ) THEN
C
C T axis.
C
                NWEIGT=6
              ELSE
              ENDIF
            ELSE IF ((NX .EQ. NY) .AND. (NX .LT. NZ)) THEN
C
C GAMMA-R-X plane.
C
              NWEIGT=24
            ELSE IF ((NY .EQ. NZ) .AND. (NZ .GT. NX)) THEN
C
C GAMMA-R-M plane.
C
              NWEIGT=24
            ELSE
            ENDIF

            WRITE(1,80) DBLE(NX)/Q,DBLE(NY)/Q,DBLE(NZ)/Q,NWEIGT
80          FORMAT('KPIN ',3F13.10,1X,I2)
            N=N+1
            IF (STRING(1:1) .GT. ' ') THEN
              WRITE(1,90) STRING
90            FORMAT(A80)
            ENDIF

100       CONTINUE
110     CONTINUE
120   CONTINUE
      WRITE(6,130) N
130   FORMAT(' Exact number of k points: ',I4)

      STOP
      END
